;********************************************************************
;
;                 JJY Radio Controlled Clock program
;
;                                       Device   : PIC16F873
;                                       Clock    : 12.8MHz
;                                       Editor   : Seiichi Inoue
;                                       Author   : Shigeyuki Yoshida
;
;             (This program can be used only in Japan)
;
;********************************************************************
;
        processor       16F873
        include         P16F873.inc
        include         picmac.inc
        list            f=INHX32
        radix           dec
        __config        _LVP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC
;
        errorlevel      -302    ;Eliminate Bank warning
        errorlevel      -307    ;Eliminate Setting page bits warning
;
;********************************************************************
;
;                          Label Definition
;
;********************************************************************
ram_base0       equ     0x20
ram_base1       equ     0xa5
;
tco_ml          equ     8       ;Lower limit for Marker
tco_mh          equ     28      ;Higher limit for Marker
tco_0l          equ     70      ;Lower limit for 0 level
tco_0h          equ     88      ;Higher limit for 0 level
tco_1l          equ     40      ;Lower limit for 1 level
tco_1h          equ     60      ;Higher limit for 1 level
tco_pl          equ     90      ;Lower limit for TCO period
tco_ph          equ     110     ;Higher limit for TCO period
;
tco_0           equ     0       ;TCO code 0
tco_1           equ     1       ;TCO code 1
tco_m           equ     2       ;TCO code mark
;
tco_vc          equ     0       ;TCO code valid bit
tco_vp          equ     1       ;TCO period valid bit
tco_err         equ     2       ;TCO error bit
;
chk_val         equ     3       ;Check count value
;
rep_start       equ     50      ;Key start delay 500mS
rep1_period     equ     30      ;Key 1 period 300mS
rep2_delay      equ     240     ;Key 2nd delay 2.4S
rep2_period     equ     10      ;Key 2 period 100mS
k_tmout_v       equ     100     ;Key time-out 10S
;
rxbflen         equ     28      ;Receive buffer length
cmd1len         equ     4       ;Grp 1 command length
cmd2len         equ     6       ;Grp 2 command length
cmd3len         equ     26      ;Grp 3 command length
;
tm1             equ     0       ;Timer 1
tm2             equ     1       ;Timer 2
tm3             equ     2       ;Timer 3
tm4             equ     3       ;Timer 4
;
;*** Key code definition
menu_key        equ     0       ;Menu key
f1_key          equ     1       ;F1 key
f2_key          equ     2       ;F2 key
f3_key          equ     3       ;F3 key
f4_key          equ     4       ;F4 key
tm_off_key      equ     5       ;Time-off key
;
;*** EEPROM address definition
ead_disp_mode   equ     0x10    ;Display mode, 24h
ead_tm1_mode    equ     0x11    ;Timer 1 mode
ead_tm1_str     equ     0x12    ;Timer 1 start time (3bytes)
ead_tm1_stop    equ     0x15    ;Timer 1 stop time  (3bytes)
ead_tm2_mode    equ     0x18    ;Timer 2 mode
ead_tm2_on      equ     0x19    ;Timer 2 start time (3bytes)
ead_tm2_off     equ     0x1c    ;Timer 2 stop time  (3bytes)
ead_tm3_mode    equ     0x1f    ;Timer 3 mode
ead_tm3_on      equ     0x20    ;Timer 3 start time (3bytes)
ead_tm3_off     equ     0x23    ;Timer 3 stop time  (3bytes)
ead_tm4_mode    equ     0x26    ;Timer 4 mode
ead_tm4_on      equ     0x27    ;Timer 4 start time (3bytes)
ead_tm4_off     equ     0x2a    ;Timer 4 stop time  (3bytes)
;
;*** Cursor position for timer setting
tm_pos0_s       equ     4       ;Mode start position
tm_pos0_e       equ     6       ;Mode end position
tm_pos1_s       equ     8       ;Hour start position
tm_pos1_e       equ     9       ;Hour end position
tm_pos2_s       equ     11      ;Minute start position
tm_pos2_e       equ     12      ;Minute end position
tm_pos3_s       equ     14      ;Second start position
tm_pos3_e       equ     15      ;Second end position
;
max_tm_index    equ     3       ;Max number of setting item
;
;*** Port symbol definition
;--- Port A
#define	timer1_out      porta_buf,0     ;Timer 1 output
#define	timer2_out      porta_buf,1     ;Timer 2 output
#define	timer3_out      porta_buf,2     ;Timer 3 output
#define	timer4_out      porta_buf,3     ;Timer 4 output
#define	rx_ok           porta_buf,5     ;Receive OK LED
;
;--- Port B
#define	port_b          portb           ;PORTB
#define	tris_b          trisb           ;TRISB
;
;--- Port C
#define	second_out      portc_buf,1     ;True second output
#define	minute_out      portc_buf,2     ;True minute output
#define	hour_out        portc_buf,3     ;True hour output
#define	day_out         portc_buf,4     ;True day output
#define	month_out       portc_buf,5     ;True month output
;
;--- Input port
#define	tco             porta,4         ;time code input
;
;--- Output port
#define	lcd_rs          port_b,0        ;LCD RS output
#define	lcd_e           port_b,1        ;LCD E output
#define	tco_out         portc,0         ;Time code output
;
;*** Flag bit definition
#define	sync_sts        dec_sts1,0      ;Second sync flag
#define	det_mark_1st    dec_sts1,1      ;1st 0 sec maker detect flag
#define	det_marker      dec_sts1,2      ;0 sec maker detect flag
#define	dec_time_1st    dec_sts1,4      ;1st time decode flag
#define	dec_time_sts    dec_sts1,5      ;Time decoded flag
#define	time_adjust     dec_sts1,6      ;Time adjust flag
;
#define	dec_minute_err  dec_sts2,0      ;Munite decode error
#define	dec_hour_err    dec_sts2,1      ;Hour decode error
#define	dec_nday_err    dec_sts2,2      ;Day decode error
#define	dec_year_err    dec_sts2,3      ;Year decode error
#define	dec_wday_err    dec_sts2,4      ;Week decode error
#define	dec_tco_err     dec_sts2,5      ;TCO code error
;
#define	disp_inh        disp_flag,0     ;Display inhibit 0:Disp/1:Stop
#define	disp_sts        disp_flag,1     ;LCD display timing flag
#define	disp_sts1       disp_flag,2     ;LCD displayed flag
#define	dec_timing      disp_flag,3     ;Decode timing flag
#define	dec_timing1     disp_flag,4     ;Decoded flag
#define	stim_out        disp_flag,5     ;Time output 0:Wait/1:Suspend
#define	tm_str_stp      disp_flag,6     ;Time set flag 0:Start/1:Stop
;
#define	d12_24          disp_mode,0     ;12/24 flag 0:24H/1:12H
;
;*** Register area definition (bank 0)
        cblock	ram_base0
        tmp_w                   ;Working register save area
        tmp_sts                 ;Status register save area
        tmp_pclath              ;Pclath register save area
        tmp_fsr                 ;FSR register save area
        tmp_fsr2                ;FSR register save area 2
        freg0                   ;Parameter register 0
        freg1                   ;Parameter register 1
        freg2                   ;Parameter register 2
        tco_code                ;TCO code 0:0, 1:1, 2:mark
        tco_valid               ;TCO valid flag
        mark_cnt                ;marker counter
        mark_pd_cnt             ;marker period counter
        zero_cnt                ;0 code counter
        dec_sts1                ;decode status1
        dec_sts2                ;decode status2
        tm_chk_cnt_wd           ;Time check counter (Week)
        tm_chk_cnt_y            ;Time check counter (Year)
        tm_chk_cnt_nd           ;Time check counter (Day)
        tm_chk_cnt_h            ;Time check counter (Hour)
        tm_chk_cnt_m            ;Time check counter (Munite)
        tm_uy                   ;Leap year flag
        tm_wd                   ;Week counter
        tm_ndh                  ;Day counter (Upper)
        tm_ndl                  ;Day counter (Lower)
        tm_10y                  ;10th of years counter
        tm_1y                   ;1st of year counter
        tm_10mth                ;10th of mounths counter
        tm_1mth                 ;1st of mounth counter
        tm_10d                  ;10th of days counter
        tm_1d                   ;1st of day counter
        tm_10h                  ;10th of hours counter
        tm_1h                   ;1st of hour counter
        tm_10m                  ;10th of minutes conuter
        tm_1m                   ;1st of minute counter
        tm_10s                  ;10th of seconds counter
        tm_1s                   ;1st of second counter
        dctm_wd                 ;Week decode counter
        dctm_100nd              ;100th of days decode counter
        dctm_10nd               ;10th of days decode counter
        dctm_1nd                ;1st of day  decode counter
        dctm_10y                ;10th of years decode counter
        dctm_1y                 ;1st of year decode counter
        dctm_10h                ;10th of hours decode counter
        dctm_1h                 ;1st of hour decode counter
        dctm_10m                ;10th of minutes decode conuter
        dctm_1m                 ;1st of minute decode counter
        bk_dctm_uy              ;Leap year backup counter
        bk_dctm_wd              ;Week backup counter
        bk_dctm_100nd           ;100th of days backup counter
        bk_dctm_10nd            ;10th of days backup counter
        bk_dctm_1nd             ;1st of day backup counter
        bk_dctm_10y             ;10th of years backup counter
        bk_dctm_1y              ;1st of year backup counter
        bk_dctm_10h             ;10th of hours backup counter
        bk_dctm_1h              ;1st of hour backup counter
        bk_dctm_10m             ;10th of minutes backup conuter
        bk_dctm_1m              ;1st of minute backup counter
        dec_min_cnt             ;Decoded time (Every minute)
        xpos                    ;X position on LCD
        ypos                    ;Y position on LCD
        acch                    ;Accumulate register A (High)
        accm                    ;Accumulate register A (Mid)
        accl                    ;Accumulate register A (Low)
        bcch                    ;Accumulate register B (High)
        bccm                    ;Accumulate register B (Mid)
        bccl                    ;Accumulate register B (Low)
        acc_flg                 ;Accumulating flag
        porta_buf               ;Port A buffer
        portc_buf               ;Port C buffer
        tled_cnt                ;LED turn on couner
        err_cnth                ;Error counter (Upper)
        err_cntl                ;Error counter (Lower)
        key_sts                 ;Key status
        key_dat                 ;Key number
        cur_timer               ;Current timer
        tm_mode                 ;Mode(0:off/1:continue/2:single)
        tm_hour                 ;Hour timer
        tm_minute               ;Munite timer
        tm_second               ;Second timer
        tm_index                ;Timer setting item index
        reserve                 ;Reserved for debugging
        k_tmout_cnt             ;Time-out counter for key proc
        tm_limit                ;Upper limit couner
        disp_flag               ;Display control flag
        disp_mode               ;Display mode
        rx_sts                  ;Receiver status
        rx_cc                   ;Received data counter
        rx_ptr                  ;Received data pointer
        unadj_cnt               ;0 second unadjust counter
        tmbf_mode               ;Timer set/check (Mode)
        tmbf_str_hour           ;Timer start set/check (Hour)
        tmbf_str_minute         ;Timer start set/check (Munite)
        tmbf_str_second         ;Timer start set/check (Second)
        tmbf_stp_hour           ;Timer stop set/check (Hour)
        tmbf_stp_minute         ;Timer stop set/check (Munite)
        tmbf_stp_second         ;Timer stop set/check (Second)
        endc
;
        cblock	ram_base1
        freg10                  ;Parameter register 10
        freg11                  ;Parameter register 11
        freg12                  ;Parameter register 12
        int_cnt                 ;Timer 2 interval for 100uS
        int_cnt10m              ;Timer 2 interval for 10mS
        int_cnt100m             ;Timer 2 interval for 100mS
        delay_cnt100u           ;100uS delay counter
        delay_cnt10m            ;10mS delay counter
        tco_cnt                 ;TCO counter
        tco_hicnt               ;TCO high counter
        tco_period              ;TCO period counter
        tco_sts                 ;TCO status
        sync_cnt                ;0 code counter for Sync detect
        ikey_dat                ;Key number
        ikey_dat0               ;Key scan position
        ikey_dat1               ;Key input data
        key_phs                 ;Key phase
        rep_cnt                 ;Key repeate counter
        eep_add                 ;EEPROM address
        eep_data                ;EEPROM data
        rx_dat:rxbflen          ;Received data buffer
        endc
;
;********************************************************************
;
;                           Program Start
;
;********************************************************************
        org     0
        lgoto   init
;
        org     4
        movwf   tmp_w           ;Save Working register
        swapf   status,w        ;Load Status register
        rbank0                  ;Set register bank 0
        movwf   tmp_sts         ;Save Status register
        movf    pclath,w        ;Load Pclath register
        movwf   tmp_pclath      ;Save Pclath register
        movf    fsr,w           ;Load FSR register
        movwf   tmp_fsr         ;Save FSR register
        lgoto   int_vec
;
;*** Message table
mes_open1:      dt      "JJY Radio Clock",0
mes_open2:      dt      "Ver.2.02",0
mes_search:     dt      "Mark searching  ",0
mes_dectime:    dt      "Time decoding   ",0
mes_wday:       dt      "SUN",0 ;Week character table
                dt      "MON",0
                dt      "TUE",0
                dt      "WED",0
                dt      "THU",0
                dt      "FRI",0
                dt      "SAT",0
;                       1234567890123456
;                       TM1 SGL 12:30:55
mes_12h:        dt      "12H",0
mes_24h:        dt      "24H",0
mes_timer       dt      "Timer",0
mes_tm          dt      "TM",0
mes_menu_tmset: dt      " "
                dt      0x7f    ;<-
                dt      "   "
                dt      0x7e    ;->
                dt      "  UP  DWN ",0
mes_start:      dt      "STR ",0
mes_stop:       dt      "STP ",0
mes_single:     dt      "SGL",0
mes_cont        dt      "CNT",0
mes_off         dt      "OFF",0
mes_nd:         dt      "ND",0
mes_unadjust:   dt      "un"
mes_adjust:     dt      "adjusted",0
mes_set:        dt      "set",0
mes_invalid     dt      "in"
mes_valid       dt      "valid",0
;
;Days for month table   1  2  3  4  5  6  7  8  9  10 11
mth_tbl:        dt      31,28,31,30,31,30,31,31,30,31,30
;
;Seria command table
cmd_tbl:        dt      "TIME"  ;Command 0 Time output
                dt      "DATE"  ;Command 1 Date output
                dt      "STIM"  ;Command 2 Time syc with Sec
                dt      "STUS"  ;Command 3 Time adj status
                dt      "STMR"  ;Command 4 Set timer
                dt      "GTMR"  ;Command 5 Output set value
                dt      "DCST"  ;Command 6 Decode status
                dt      "TMON"  ;Command 7 Timer ON
                dt      "TMOF"  ;Command 8 Timer OFF
                dt      0       ;End of command
;
cmdtime         equ     0       ;Command 0
cmddate         equ     1       ;Command 1
cmdstim         equ     2       ;Command 2
cmdstus         equ     3       ;Command 3
cmdstmr         equ     4       ;Command 4
cmdgtmr         equ     5       ;Command 5
cmddecd         equ     6       ;Command 6
cmdtmon         equ     7       ;Command 7
cmdtmof         equ     8       ;Command 8
;
;Upper limit value of timer menu
tm_lim_tbl:     dt      2       ;Mode
                dt      23      ;Hour
                dt      59      ;Munite
                dt      59      ;Second
;
;####################################################################
;
;                    PIC LCD display subroutine
;
;####################################################################
max_cl          equ     16      ;Digit number of LCD
max_ln          equ     2       ;Line number of LCD
;
;********************************************************************
; Read data from data table
;    Input parameter   freg1 = Data address
;    Output parameter  w = Data
;    Used register     freg1
;********************************************************************
get_tbl_data:
        movf    freg1,w         ;Read address
        setpch                  ;PCLATH setting macro
        movwf   pcl             ;Jump to data read
;
;
;********************************************************************
; Initialization of the LCD module
;    Input parameter   None
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
lcd_init:
        movlw   2
        call    delay10m        ;20mS delay
        movlw   b'00000010'
        andwf   port_b,f        ;Clear except E bit
        movlw   b'00110000'     ;DB4,DB5 = 1
        iorwf   port_b,f        ;Set function to 8 bit interface
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        ei                      ;Enable interrupt

        movlw   80
        call    delay100u       ;8.2mS delay
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        ei                      ;Enable interrupt

        movlw   2
        call    delay100u       ;200uS delay
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        ei                      ;Enable interrupt

        movlw   b'00000010'
        andwf   port_b,f        ;Clear except E bit
        movlw   b'00100000'     ;DB5 = 1
        iorwf   port_b,f        ;Set function to 4 bit interface

        movlw   2
        call    delay100u       ;200uS delay
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        ei                      ;Enable interrupt

        movlw   2
        call    delay100u       ;200uS delay
        movlw   b'00101000'     ;0010+1000
        call    lcd_iwr         ;Set 4 bit, 2 line, 5x7 dot
        movlw   b'00000110'     ;0000+0110
        call    lcd_iwr         ;Set entry mode

        movlw   b'10000000'     ;1000+0000
        call    lcd_iwr         ;Move cursor to home position
        movlw   b'00010100'     ;0001+0100
        call    lcd_iwr         ;Shift cursor to right
        movlw   b'00001100'     ;0000+1100
        call    lcd_iwr         ;Display=ON, Cursor=OFF, Blink=OFF

        call    cls             ;Clear display
        return
;
;
;********************************************************************
; Output 0.625uS pulse to E ternminal of LCD
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
lcd_e_pulse:
        bsf     lcd_e           ;PORTB bit1(E) ON
        nop                     ;312.5nS
        bcf     lcd_e           ;PORTB bit1(E) OFF
        return
;
;
;********************************************************************
; One letter display
;    Input parameter   w = display code
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
print:
        call    lcd_dwr         ;Write display code to LCD
        incf    xpos,f          ;X position + 1
        cp      xpos,max_cl     ;X position is max ?
        jpnz    print2          ;Not max. Jump to print2
        clrf    xpos            ;Max. Clear X position
        incf    ypos,f          ;Y position + 1
        cp      ypos,max_ln     ;Y position is max ?
        skpnz                   ;If not max, skip next line
        clrf    ypos            ;Clear Y position
print1:
        call    locate          ;Change cursor location
print2:
        return
;
;
;********************************************************************
; Message display
;    Input parameter   w = display message address
;    Output parameter  None
;    Used register     freg0,freg1,xpos,ypos
;********************************************************************
messout:
        movwf   freg1           ;Set message address
messout1:
        call    get_tbl_data    ;Read message
        iorlw   0               ;w = 0 ? ( end ? )
        skpnz                   ;If not end, skip next line
        goto    messout2        ;Jump to return
        call    lcd_dwr         ;Write data to LCD
        incf    xpos,f          ;X position + 1
        cp      xpos,max_cl     ;X positon is max ?
        jpnz    messout3        ;Not max. Jump to messout3
        clrf    xpos            ;Max. Clear X position
        incf    ypos,f          ;Y position + 1
        cp      ypos,max_ln     ;Y position is max ?
        jpnz    messout4        ;Not max. Jump to messout4
        clrf    ypos            ;Clear Y position
messout4:
        call    locate          ;Change cursor location
messout3:
        incf    freg1,f         ;Message address + 1
        goto    messout1        ;Next data
messout2:
        return
;
;
;********************************************************************
; Clear display
;    Input parameter   None
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
cls:
        clrf    xpos            ;Clear X position
        clrf    ypos            ;Clear Y position
        movlw   b'00000001'     ;0000+0001
        call    lcd_iwr         ;Clear display
        movlw   30
        call    delay100u       ;3.1mS delay
        return
;
;
;********************************************************************
; Change cursor location
;    Input parameter   xpos = X position ( 0 - 15 )
;                      ypos = Y position ( 0 - 1 )
;    Output parameter  None
;    Used register     freg0
;********************************************************************
locate:
        movf    xpos,w          ;Read X position
        movf    ypos,f          ;Y position = 0 ?
        skpz                    ;If Y=0, skip next line
        iorlw   b'01000000'     ;Y=1.
        iorlw   b'10000000'     ;Location indicate command
        call    lcd_iwr         ;Set cursor indication
        movlw   30
        call    delay100u       ;3.1mS delay
        return
;
;
;********************************************************************
; Cursor indication
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
csr_on:
        movlw   b'00001110'
        call    lcd_iwr         ;Cursor indication
        return
;
;
;********************************************************************
; Cursor non-indication
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
csr_off:
        movlw   b'00001100'
        call    lcd_iwr         ;Cursor non-indication
        return
;
;
;********************************************************************
; Write data to LCD instruction register
;    Input parameter   w = data
;    Output parameter  None
;    Used register     freg0
;********************************************************************
lcd_iwr:
        movwf   freg0           ;Save data
        andlw   b'11110000'     ;Pick up MSB 4 bits
        movwf   port_b          ;Set data to PORTB
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        swapf   freg0,w         ;Swap upper and lower 4 bits
        andlw   b'11110000'     ;Pick up LSB 4 bits
        movwf   port_b          ;Set data to PORTB
        call    lcd_e_pulse     ;Send a pulse to LCD E term
        ei                      ;Enable interrupt
        movlw   2
        call    delay100u       ;200uS delay
        return
;
;
;********************************************************************
; Write data to LCD data register
;    Input parameter   w = data
;    Output parameter  None
;    Used register     freg0
;********************************************************************
lcd_dwr:
        movwf   freg0           ;Save data
        andlw   b'11110000'     ;Pick up MSB 4 bits
        movwf   port_b          ;Set data to PORTB
        bsf     lcd_rs          ;Set RS = 1
        di                      ;Disable interrupt
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        swapf   freg0,w         ;Swap upper and lower 4 bits
        andlw   b'11110000'     ;Pick up LSB 4 bits
        movwf   port_b          ;Set data to PORTB
        bsf     lcd_rs          ;Set RS = 1
        call    lcd_e_pulse     ;Send a pulse to LCD E terminal
        ei                      ;Enable interrupt
        movlw   2
        call    delay100u       ;200uS delay
        return
;
;
;********************************************************************
; Second display
;    Input parameter   tm_1s,tm_10s
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_second:
        movf    tm_10s,w        ;Read 10th of second
        addlw   '0'             ;Change to character code
        call    print           ;10th of second display
        movf    tm_1s,w         ;Read 1st of second
        addlw   '0'             ;Change to character code
        call    print           ;1st of second display
        return
;
;
;********************************************************************
; Minite display
;    Input parameter   tm_1m,tm_10m
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_minute:
        movf    tm_10m,w        ;Read 10th of minite
        addlw   '0'             ;Change to character code
        call    print           ;10th of minite dsiplay
        movf    tm_1m,w         ;Read 1st of minite
        addlw   '0'             ;Change to character code
        call    print           ;1st of minite display
        return
;
;
;********************************************************************
; Hour display
;    Input parameter   tm_1h,tm_10h
;    Output parameter  None
;    Used register     freg0,freg1
;********************************************************************
disp_hour:
        btfss   d12_24          ;24 hour mode ? (0:24H/1:12H)
        goto    disp_hour3      ;Yes. Jump to 24 hour process
;
;12 hour type
        clrf    bcch            ;Clear output buffer
        movf    tm_1h,w         ;Read 1st of hour
        movwf   bccl            ;Set 1st of hour data
        movf    tm_10h,w        ;Read 10th of hour
        movwf   accl            ;Set 10th of hour data
        movlw   10
        call    mulib           ;10th * 10
        movf    bccl,w          ;Read lower 1st of hour data
        movwf   accl            ;Save
        movlw   12
        subwf   accl,w          ;After 12 hour ?
        jpb     disp_hour1      ;If no, jump to disp_hour1
        movwf   accl            ;
        movlw   'P'             ;Set character "P"
        goto    disp_hour2
disp_hour1:
        movlw   'A'             ;Set character "A"
disp_hour2:
        call    print           ;"A" or "P" display
        movlw   10
        call    divb            ;accl / w
        movlw   '0'             ;Set "0" character code
        addwf   acch,w          ;Change to character code
        call    print           ;10th of hour display
        movlw   '0'             ;Set "0" character code
        addwf   accl,w          ;Change to character code
        call    print           ;1st of hour display
        goto    disp_hour4
;
;24 hour type
disp_hour3:
        movlw   ' '             ;Set blanc character code
        call    print           ;Blanc display
        movf    tm_10h,w        ;Read 10th of hour
        addlw   '0'             ;Change to character code
        call    print           ;10th of hour display
        movf    tm_1h,w         ;Read 1st of hour
        addlw   '0'             ;Change to character code
        call    print           ;1st of hour display
disp_hour4:
        return
;
;
;********************************************************************
; Day display
;    Input parameter   tm_1d,tm_10d
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_day:
        movf    tm_10d,w        ;Read 10th of day
        addlw   '0'             ;Change to character code
        call    print           ;10th of day display
        movf    tm_1d,w         ;Read 1st of day
        addlw   '0'             ;Change to character code
        call    print           ;1st of day display
        return
;
;
;********************************************************************
; Month display
;    Input parameter   tm_1mth,tm_10mth
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_month:
        movf    tm_10mth,w      ;Read 10th of month
        addlw   '0'             ;Change to character code
        call    print           ;10th of month display
        movf    tm_1mth,w       ;Read 1st of month
        addlw   '0'             ;Change to character code
        call    print           ;1st of month display
        return
;
;
;********************************************************************
; Year display
;    Input parameter   tm_1y,tm_10y
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_year:
        movlw   "'"             ;Set Quotation mark(')
        call    print           ;Quotation mark display
        movf    tm_10y,w        ;Read 10th of year
        addlw   '0'             ;Change to character code
        call    print           ;10th of year display
        movf    tm_1y,w         ;Read 1st of year
        addlw   '0'             ;Change to character code
        call    print           ;1st of year display
        return
;
;
;********************************************************************
; Week display
;    Input parameter   tm_wd
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_wday:
        clrc                    ;Clear Carry bit
        rlf     tm_wd,w         ;w = tm_wd * 2
        movwf   freg0           ;Save data
        rlf     freg0,w         ;w = tm_wd * 4
        addlw   mes_wday        ;Set week message table address
        call    messout         ;Week message display
        return
;
;
;********************************************************************
; Normalized day display
;    Input parameter   tm_ndh,tm_ndl
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_nday:
        clrf    freg0           ;Clear work register
        movf    tm_ndh,w        ;Read upper part of total day
        movwf   acch            ;Set upper data
        movf    tm_ndl,w        ;Read lower part of total day
        movwf   accl            ;Set lower data
disp_nday1:
        clrf    bcch            ;Clear work register
        movlw   100
        movwf   bccl            ;Set index
        call    us_subw         ;acch,accl=acch,accl-bcch,bccl(100)
        movf    acc_flg,f       ;Check flag 1:could / 0:couldn't
        jpz     disp_nday2      ;End of calculation
        incf    freg0,f         ;Result counter + 1
        goto    disp_nday1      ;Again
disp_nday2:
        movf    freg0,w         ;Read 100th data
        addlw   '0'             ;Change to character code
        call    print           ;100th of normalized day display
        movlw   10
        call    divb            ;accl / w
        movf    acch,w          ;Read quotient
        addlw   '0'             ;Change to character code
        call    print           ;10th of normalized day display
        movf    accl,w          ;Read surplus
        addlw   '0'             ;Change to character code
        call    print           ;1st of normalized day display
        return
;
;
;********************************************************************
; Time display
;    Input parameter   tm_1s,tm_10s,tm_1m,tm_10m,tm1h,tm_10h
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_time:
        clrf    xpos            ;Clear X position
        clrf    ypos            ;Clear Y position
        incf    ypos,f          ;Set 2nd line
        call    locate          ;Change cursor location
        call    disp_hour       ;Hour display
        movlw   ':'             ;Set colon(:)
        call    print           ;Colon display
        call    disp_minute     ;Minute display
        movlw   ':'             ;Set colon(:)
        call    print           ;Colon display
        call    disp_second     ;Second display
        movlw   10
        movwf   xpos            ;Set X position
        incf    ypos,f          ;Y will be over ? (No affection here)
        call    locate          ;Change cursor location
        call    disp_nday       ;Normalized day display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        movlw   mes_nd          ;Set "ND" message table address
        call    messout         ;"ND" message display
        return
;
;
;********************************************************************
; Date display
;    Input parameter   tm_1d,tm_10d,tm_1mth,tm_10mth,tm_wd,
;                      tm_1y,tm_10y,tm_ndh,tm_ndl
;    Output parameter  None
;    Used register     freg0
;********************************************************************
disp_date:
        clrf    xpos            ;Clear X position
        clrf    ypos            ;Clear Y position
        call    locate          ;Change cursor location
        call    disp_year       ;Year display
        movlw   '/'             ;Set slash(/)
        call    print           ;Slash display
        call    disp_month      ;Month display
        movlw   '/'             ;Set slash(/)
        call    print           ;Slash display
        call    disp_day        ;Day display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        call    disp_wday       ;Week display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        return
;
;
;********************************************************************
; Normal reception display
;    Input parameter   None
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
disp_ok:
        bsf     rx_ok           ;Light up receiving LED
        movlw   15
        movwf   xpos            ;Set X position
        clrf    ypos            ;Clear Y position
        call    locate          ;Change cursor location
        movlw   'O'             ;Set OK status(0)
        call    print           ;OK status display
        return
;
;
;********************************************************************
; No reception display
;    Input parameter   None
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
disp_ng:
        bcf     rx_ok           ;Turn off receiving LED
        movlw   15
        movwf   xpos            ;Set X position
        clrf    ypos            ;Clear Y position
        call    locate          ;Change cursor location
        movlw   'X'             ;Set NG status(X)
        call    print           ;NG status display
        return
;
;
;********************************************************************
; 1st line of timer set display
;    Input parameter   tm_hour,tm_minute,tm_second
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
disp_timer:
        movlw   mes_tm          ;Set "TM" message table address
        call    messout         ;"TM" display
        movf    cur_timer,w     ;Read current timer
        addlw   '1'             ;Change to character code
        call    print           ;Current timer number display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        movlw   mes_off         ;Set "OFF" message table address
        movf    tm_mode,f       ;Check timer mode (0:off)
        jpz     disp_timer1     ;If mode=off, jump to disp_timer1
        movlw   mes_cont        ;Set "CNT" message table address
        btfss   tm_mode,0       ;Continue mode ? (1:continue)
        movlw   mes_single      ;Set "SGL" message table address
disp_timer1:
        call    messout         ;Mode message display
        movlw   ' '             ;Set blanc
        call    print           ;Blanc display
        movf    tm_hour,w       ;Read Hour timer
        movwf   accl            ;Set Hour value to ACCL
        movlw   10
        call    divb            ;Hour/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of hour display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of hour display
        movlw   ':'
        call    print           ;Colon(:) display
        movf    tm_minute,w     ;Read Munite timer
        movwf   accl            ;Set Munite value to ACCL
        movlw   10
        call    divb            ;Munite/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of Munite display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of Munite display
        movlw   ':'
        call    print           ;Colon(:) display
        movf    tm_second,w     ;Read Second timer
        movwf   accl            ;Set Second value to ACCL
        movlw   10
        call    divb            ;Second/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of Second display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of Second display
        return
;
;
;********************************************************************
; Time set value display
;    Input parameter   tm_index=index of time set item
;    Output parameter  None
;    Used register     freg0,xpos,ypos
;********************************************************************
disp_tm_set:
        clrf    ypos            ;Clear Y position
        movf    tm_index,w      ;Check index
        jpnz    disp_tm_set2    ;If index not 0, jump to disp_tm_set2
        movlw   tm_pos0_s       ;Read mode start position
        movwf   xpos            ;Set mode start position
        call    locate          ;Change cursor location
        movlw   mes_off         ;Set "OFF" message table address
        movf    tm_mode,f       ;Check timer mode (0:off)
        jpz     disp_tm_set1    ;If mode=off, jump to disp_tm_set1
        movlw   mes_cont        ;Set "CNT" message table address
        btfss   tm_mode,0       ;Continue mode ? (1:continue)
        movlw   mes_single      ;Set "SGL" message table address
disp_tm_set1:
        call    messout         ;Mode message display
        movlw   tm_pos0_s       ;Read mode start position
        goto    disp_tm_set5    ;Jump to disp_tm_set5
disp_tm_set2:	
        cp      tm_index,1      ;Check index
        jpnz    disp_tm_set3    ;If index not 1, jump to disp_tm_set3
        movlw   tm_pos1_s       ;Set hour set position
        movwf   xpos            ;Set hour set position to XPOS
        call    locate          ;Change cursor location
        movf    tm_hour,w       ;Read hour timer
        movwf   accl            ;Set hour value to ACCL
        movlw   10
        call    divb            ;Hour/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of hour display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of hour display
        movlw   tm_pos1_s       ;Set hour set position
        goto    disp_tm_set5    ;Jump to disp_tm_set5
disp_tm_set3:
        cp      tm_index,2      ;Check index
        jpnz    disp_tm_set4    ;If index not 2, jump to disp_tm_set4
        movlw   tm_pos2_s       ;Set munite set position
        movwf   xpos            ;Set munite set position to XPOS
        call    locate          ;Change cursor location
        movf    tm_minute,w     ;Read munite timer
        movwf   accl            ;Set munite value to ACCL
        movlw   10
        call    divb            ;Munite/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of munite display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of munite display
        movlw   tm_pos2_s       ;Set munite set position
        goto    disp_tm_set5    ;Jump to disp_tm_set5
disp_tm_set4:
        movlw   tm_pos3_s       ;Set second set position
        movwf   xpos            ;Set second set position to XPOS
        call    locate          ;Change cursor location
        movf    tm_second,w     ;Read second timer
        movwf   accl            ;Set second value to ACCL
        movlw   10
        call    divb            ;Second/10
        movlw   '0'
        addwf   acch,w          ;Change to character code
        call    print           ;10th of second display
        movlw   '0'
        addwf   accl,w          ;Change to character code
        call    print           ;1st of second display
        movlw   tm_pos3_s       ;Set second set position
disp_tm_set5:
        movwf   xpos            ;Set X position
        clrf    ypos            ;Clear Y position
        call    locate          ;Change cursor location
        call    csr_on          ;Display cursor
        return
;
;
;********************************************************************
; Increment 1 second
;    Input parameter   tm_1s,tm_10s,tm_1m,tm_10m,tm_1h,tm_10h,
;                      tm_1d,tm_10d,tm_1mth,tm_10mth,tm_1y,tm_10y,
;                      tm_wd,tm_nd
;    Output parameter  tm_1s,tm_10s,tm_1m,tm_10m,tm_1h,tm_10h,
;                      tm_1d,tm_10d,tm_1mth,tm_10mth,tm_1y,tm_10y,
;                      tm_wd,tm_nd
;    Used register     tm_uy,acch,accl,bcch,bccl,acc_flg
;********************************************************************
inc_time:
        clrf    portc_buf       ;Clear Port C buffer
        clrf    tled_cnt        ;Clear LED turn on counter
        bsf     second_out      ;Set true second LED bit
        incf    tm_1s,f         ;1st of second counter + 1
        cp      tm_1s,10        ;1st of second counter = 10 ?
        jpnz    inc_time2       ;No. Jump to return
        clrf    tm_1s           ;Yes. Clear 1st of second counter
        incf    tm_10s,f        ;10th of second counter + 1
        cp      tm_10s,6        ;10th of second counter = 6 ?
        jpnz    inc_time2       ;No. Jump to return
        bsf     minute_out      ;Yes. Set true minute LED bit
        clrf    tm_10s          ;Clear 10th of second counter
        btfsc   dec_time_sts    ;Time decoded ?
        bcf     sync_sts        ;Yes. Clear Second sync flag
        incf    tm_1m,f         ;No. 1st of munite counter + 1
        cp      tm_1m,9         ;1st of munite counter = 9 ?
        jpnz    inc_time0       ;No. Jump to inc_time0
        incf    unadj_cnt,f     ;Yes. 0 second unadjust counter + 1
        cp      unadj_cnt,7     ;Unadjust counter > 7 ?
        jpb     inc_time02      ;If counter =< 6, jump to inc_time02
        movlw   6
        movwf   unadj_cnt       ;Set unadjust counter = 6
inc_time02:
        btfsc   dec_time_sts    ;Time decoded ?
        goto    inc_time00      ;Yes. Skip Error conter incrementation
        incf    err_cntl,f      ;No. Lower error counter + 1
        jpnz    inc_time00      ;Overflow ?
        incf    err_cnth,f      ;Yes. Higher error counter + 1
inc_time00:
                                ;Upper limit of Error counter is 999
        cp      err_cnth,3      ;Higher error counter = 3 ?
        jpnz    inc_time01      ;No. Jump to inc_time01
        cp      err_cntl,0xe8   ;Yes. Lower error counter = h'E8' ?
        jpnz    inc_time01      ;No. Jump to inc_time01
        decf    err_cntl,f      ;Yes. Lower error counter -1
inc_time01:
        bcf     time_adjust     ;Clear time adjust flag
        bsf     sync_sts        ;Set Second sync flag
        bsf     det_marker      ;Set 0 second maker detect flag
        bsf     dec_time_sts    ;Set time decoded flag
inc_time0:
        cp      tm_1m,10        ;1st munite counter = 10 ?
        jpnz    inc_time2       ;No. Jump to return
        clrf    tm_1m           ;Yes. Clear 1st munite counter
        bsf     time_adjust     ;Set time adjust flag
        bcf     sync_sts        ;Clear second sync flag
        bcf     det_marker      ;Clear 0 second maker detect flag
        bcf     dec_time_sts    ;Clear time decoded flag
        incf    tm_10m,f        ;10th of munite counter + 1
        cp      tm_10m,6        ;10th of munite counter = 6 ?
        jpnz    inc_time2       ;No. Jump to return
        bsf     hour_out        ;Yes. Set true hour LED bit
        clrf    tm_10m          ;Clear 10th of munite counter
        incf    tm_1h,f         ;1st of hour counter + 1
        cp      tm_1h,10        ;1st of hour counter = 10 ?
        jpnz    inc_time1       ;No. Jump to inc_time1
        clrf    tm_1h           ;Yes. Clear 1st of hour counter
        incf    tm_10h,f        ;10th of hour counter + 1
        goto    inc_time2       ;Jump to return
inc_time1:
        cp      tm_10h,2        ;10th of hour counter = 2 ?
        jpnz    inc_time2       ;No. Jump to return
        cp      tm_1h,4         ;Yes. 1st of hour counter = 4 ?
        jpnz    inc_time2       ;No. Jump to return
        clrf    tm_1h           ;Clear 1st of hour counter
        clrf    tm_10h          ;Clear 10th of hour counter
        bsf     day_out         ;Set true day LED bit
        call    inc_date        ;Day + 1
inc_time2:
        return
;
;
;********************************************************************
; Increment 1 day
;    Input parameter   tm_1d,tm_10d,tm_1mth,tm_10mth,tm_1y,tm_10y,
;                      tm_wd,tm_nd
;    Output parameter  tm_1d,tm_10d,tm_1mth,tm_10mth,tm_1y,tm_10y,
;                      tm_wd,tm_nd
;    Used register     tm_uy,acch,accl,bcch,bccl,acc_flg
;********************************************************************
inc_date:
        clrf    tm_uy           ;Clear leap year flag
        movlw   b'00000001'     ;Set mask
        andwf   tm_10y,w        ;Pick up LSB
        jpz     inc_date1       ;If even number, jump to inc_date1
        movlw   2
inc_date1:
        addwf   tm_1y,w
        andlw   b'00000011'     ;Check leap year
        skpnz                   ;Leap year ?
        incf    tm_uy,f         ;Yes. Set leap year flag
        incf    tm_wd,f         ;No. Week data + 1
        cp      tm_wd,7         ;Week data = 7 ?
        jpnz    inc_date2       ;No. Jump to inc_date2
        clrf    tm_wd           ;Yes. Clear week data
inc_date2:
        movlw   high 365        ;Pickup upper byte of h'16d'(365)
        movwf   acch            ;Set upper byte = h'1' to ACCH
        movlw   low 365+1       ;Set lower byte = h'6e'
        movf    tm_uy,f         ;Leap year ?
        jpz     inc_date3       ;No. Jump to inc_date3
        movlw   low 366+1       ;Yes. Set lower byte = h'6f'
inc_date3:
        movwf   accl            ;Set lower byte to ACCL
        incf    tm_ndl,f        ;Lower normalized day + 1
        jpnz    inc_date4       ;Overflow ? If no, skip next step
        incf    tm_ndh,f        ;Upper normalized day + 1
inc_date4:
        movf    tm_ndh,w        ;Read upper normalized day
        movwf   bcch            ;Set upper normalized day to BCCH
        movf    tm_ndl,w        ;Read lower normalized day
        movwf   bccl            ;Set lower normalized day to BCCL
        call    us_subw         ;ACCH,ACCL = ACCH,ACCL - BCCH,BCCL
        movf    acch,f          ;Normalized day overflow ?
        jpnz    inc_date5       ;No. Jump to inc_date5
        movf    accl,f          ;Yes. Normalized day overflow ?
        jpnz    inc_date5       ; No. Jump to inc_date5
                                ; Yes. Overflow
        clrf    tm_ndh          ;Clear upper normalized day
        clrf    tm_ndl          ;Clear lower normalized day
        incf    tm_ndl,f        ;Set normalized day to 1
        incf    tm_1y,f         ;1st of year data + 1
        cp      tm_1y,10        ;1st of year data = 10 ?
        jpnz    inc_date5       ;No. Jump to inc_date5
        clrf    tm_1y           ;Yes. Clear 1st of year data
        incf    tm_10y,f        ;10th of year data + 1
        cp      tm_10y,10       ;10th of year data = 10 ?
        jpnz    inc_date5       ;No. Avalable upto 2099 year
        clrf    tm_10y          ;Yes. Clear data but not correct year
inc_date5:
        call    cvt_nd2day      ;Convert normalized day to month/day
        cp      tm_1d,1         ;1st day ?
        jpnz    inc_date6       ;No. Jump to return
        movf    tm_10d,f        ;Yes. 10th of month = 0 ?
        jpnz    inc_date6       ; No. Jump to return
        bsf     month_out       ; Set true month LED bit
inc_date6:
        return
;
;
;********************************************************************
; Increment backup 1 munite
;    Input parameter   bk_dctm_1m,bk_dctm_10m,bk_dctm_1h,bk_dctm_10h,
;                      bk_dctm_1nd,bk_dctm_10nd,bk_dctm_100nd,
;                      bk_dctm_1y,bk_dctm_10y,bk_dctm_wd
;    Output parameter  bk_dctm_1m,bk_dctm_10m,bk_dctm_1h,bk_dctm_10h,
;                      bk_dctm_1nd,bk_dctm_10nd,bk_dctm_100nd,
;                      bk_dctm_1y,bk_dctm_10y,bk_dctm_wd,bk_dctm_uy
;    Used register     tm_uy,acch,accl,bcch,bccl,acc_flg
;********************************************************************
inc_dec_time:
        movf    bk_dctm_10y,w   ;Read 10th of years backup counter
        movwf   bk_dctm_uy      ;Save Leap year backup counter
        rlf     bk_dctm_uy,f    ;10th of years backup counter x 2
        movlw   b'00000010'     ;Set mask
        andwf   bk_dctm_uy,f    ;Save LSB of 10th of years x 2
        movf    bk_dctm_1y,w    ;Read 1st of year backup counter
        addwf   bk_dctm_uy,w    ;1st of year + (10th of years x 2)
        clrf    bk_dctm_uy      ;Clear Leap year flag
        andlw   3               ;Leap year ?
        skpnz                   ;No. Skip next line
        bsf     bk_dctm_uy,0    ;Yes. Set Leap year flag
        incf    bk_dctm_1m,f    ;1st of munite + 1
        cp      bk_dctm_1m,10   ;1st of munite = 10 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        clrf    bk_dctm_1m      ;Yes. Clear 1st of munite
        incf    bk_dctm_10m,f   ;10th of munite + 1
        cp      bk_dctm_10m,6   ;10th of munite = 6 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        clrf    bk_dctm_10m     ;Yes. Clear 10th of munite
        incf    bk_dctm_1h,f    ;1st of hour + 1
        cp      bk_dctm_1h,10   ;1st of hour = 10 ?
        jpnz    inc_dec_time1   ;No. Jump to inc_dec_time1
        clrf    bk_dctm_1h      ;Yes. Clear 1st of hour
        incf    bk_dctm_10h,f   ;10th of hour + 1
        goto    inc_dec_time4   ;Jump to return
inc_dec_time1:
        cp      bk_dctm_10h,2   ;10th of hour = 2 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        cp      bk_dctm_1h,4    ;Yes. 1st of hour = 4 ?
        jpnz    inc_dec_time4   ; No. Jump to return
        clrf    bk_dctm_1h      ; Yes. Clear 1st of hour
        clrf    bk_dctm_10h     ;Clear 10th of hour
        incf    bk_dctm_wd,f    ;Week + 1
        cp      bk_dctm_wd,7    ;Week = 7 ?
        jpnz    inc_dec_time2   ;No. Jump to inc_dec_time2
        clrf    bk_dctm_wd      ;Yes. Clear week
inc_dec_time2:
        incf    bk_dctm_1nd,f   ;1st of normalized day + 1
        cp      bk_dctm_1nd,10  ;1st of normalized day = 10 ?
        jpnz    inc_dec_time3   ;No. Jump to inc_dec_time3
        clrf    bk_dctm_1nd     ;Yes. Clear 1st of normalized day
        incf    bk_dctm_10nd,f  ;10th of normalized day + 1
        cp      bk_dctm_10nd,10 ;10th of normalized day = 10 ?
        jpnz    inc_dec_time3   ;No. Jump to inc_dec_time3
        clrf    bk_dctm_10nd    ;Yes. Clear 10th of normalized day
        incf    bk_dctm_100nd,f ;100th of normalized day + 1
        goto    inc_dec_time4   ;Jump to return
inc_dec_time3:
        cp      bk_dctm_100nd,3 ;100th of normalized day = 3 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        cp      bk_dctm_10nd,6  ;Yes. 10th of normalized day = 6 ?
        jpnz    inc_dec_time4   ; No. Jump to return
        movlw   6               ; Yes. Set 1st of normalized day (6)
        movf    bk_dctm_uy,f    ;Leap year ?
        skpnz                   ;No. Skip next line
        addlw   1               ;Yes. Add one day (7)
        cpw     bk_dctm_1nd     ;1st of normalized day overflow ?
        jpnz    inc_dec_time4   ;No. Jump to return
        clrf    bk_dctm_1nd     ;Yes. Clear 1st of normalized day
        incf    bk_dctm_1nd,f   ;Set 1st of normalized day = 1
        clrf    bk_dctm_10nd    ;Clear 10th of normalized day
        clrf    bk_dctm_100nd   ;Clear 100th of normalized day
        incf    bk_dctm_1y,f    ;1st of year + 1
        cp      bk_dctm_1y,10   ;1st of year = 10 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        clrf    bk_dctm_1y      ;Yes. Clear 1st of year
        incf    bk_dctm_10y,f   ;10th of year + 1
        cp      bk_dctm_10y,10  ;10th of year = 10 ?
        jpnz    inc_dec_time4   ;No. Jump to return
        clrf    bk_dctm_10y     ;Yes. 10th of year but it's not correct year
inc_dec_time4:
        return
;
;
;********************************************************************
; Convert normalized day to month and day data
;    Input parameter   tm_ndl,tm_ndh,tm_uy
;    Output parameter  tm_1d,tm_10d,tm_1mth,tm_10mth
;    Used register     acch,accl,bcch,bccl,acc_flg
;********************************************************************
cvt_nd2day:
        movlw   mth_tbl         ;Read month day data table addess
        movwf   freg1           ;Save addess as an input parameter
        clrf    tm_1mth         ;Clear 1st of mounth
        clrf    tm_10mth        ;Clear 10th of mounth
        clrf    bcch            ;Clear BCCH register
        clrf    bccl            ;Clear BCCL register
        movf    tm_ndh,w        ;Read upper normalized day
        movwf   acch            ;Set upper normalized day to ACCH
        movf    tm_ndl,w        ;Read lower normalized day
        movwf   accl            ;Set upper normalized day to ACCL
cvt_nd2day1:
        incf    tm_1mth,f       ;1st of mounth + 1
        cp      tm_1mth,12      ;1st of mounth = 12 ?
        jpz     cvt_nd2day4     ;Yes. Jump to cvt_nd2day4
        call    get_tbl_data    ;No. Read Month day data (W)
        incf    freg1,f         ;1st of mounth input + 1
        movwf   bccl            ;Set Month day data to BCCL
        clrf    bcch            ;Clear BCCH register
cvt_nd2day2:
        movf    tm_uy,f         ;Leap year ?
        jpz     cvt_nd2day3     ;Yes. Jump to cvt_nd2day3
        cp      tm_1mth,2       ;No. Mounth = February ?
        jpnz    cvt_nd2day3     ; No. Jump to cvt_nd2day3
        incf    bccl,f          ; Yes. Month day on BCCL + 1
cvt_nd2day3:
        call    us_subw         ;ACCH,ACCL = ACCH,ACCL - BCCH,BCCL
        movf    acc_flg,f       ;Subtraction success ?
        jpz     cvt_nd2day4     ;No. Jump to cvt_nd2day4
        movf    acch,f          ;Yes. upper normalized day = 0 ?
        jpnz    cvt_nd2day1     ; No. Jump to cvt_nd2day1
        movf    accl,f          ; Yes. lower normalized day = 0 ?
        jpnz    cvt_nd2day1     ;  No. Jump to cvt_nd2day1
        movf    bccl,w          ;  Yes. Read Month day data from BCCL
        movwf   accl            ;Set Month day data to ACCL
cvt_nd2day4:
        movf    acch,f          ;ACCH(Upper normalized day) = 0 ?
        jpnz    cvt_nd2day5     ;No. Jump to cvt_nd2day5(Error)
        cp      accl,31+1       ;Yes. Lower normalized day >= 32 ?
        jpnb    cvt_nd2day5     ; Yes. Jump to cvt_nd2day5(Error)
        movlw   10              ; No.
        call    divb            ;ACCL/10
        movf    acch,w          ;Read 10th of day
        movwf   tm_10d          ;Set 10th of day
        movf    accl,w          ;Read 1st of day
        movwf   tm_1d           ;Set 1st of day
        cp      tm_1mth,10      ;1st of mounth >= 10 ?
        jpb     cvt_nd2day6     ;No. Jump to return
        movlw   10              ;Yes.
        subwf   tm_1mth,f       ;1st of mounth - 10
        incf    tm_10mth,f      ;10th of mounth + 1
        goto    cvt_nd2day6     ;Jump to return
cvt_nd2day5:
        clrf    tm_1d           ;Clear 1st of day
        clrf    tm_10d          ;Clear 10th of day
        clrf    tm_1mth         ;Clear 1st of mounth
        clrf    tm_10mth        ;Clear 10th of mounth
cvt_nd2day6:
        return
;
;
;********************************************************************
; Addition  [ BCCH,BCCL = BCCH,BCCL + ACCH,ACCL ]
;    Input parameter   acch,accl = Addend
;                      bcch,bccl = Augend
;    Output parameter  bcch,bccl = Summation
;    Used register     acch,accl,bcch,bccl
;********************************************************************
addw:
        movf    accl,w          ;Read ACCL
        addwf   bccl,f          ;BCCL + ACCL
        skpnc                   ;If not overflow, skip next line
        incf    bcch,f          ;BCCH + 1
        movf    acch,w          ;Read ACCH
        addwf   bcch,f          ;BCCH + ACCH
        return
;
;
;********************************************************************
; Multiplication [ BCCH,BCCL = BCCH,BCCL + (ACCL * W) ]
;    Input parameter   bcch,bccl = Augend
;                      accl = Multiplicand
;                      W = Multiplier
;    Output parameter  bcch,bccl = Product
;    Used register     acch,accm,accl,bcch,bccm,bccl
;********************************************************************
mulib:
        clrf    acch            ;Clear ACCH
        movwf   accm            ;Save Multiplier to ACCM
        movlw   8               ;Set loop count value
        movwf   bccm            ;Set loop counter
mulib1:
        rrf     accm,f          ;Check '1' of Multiplier bit
        skpnc                   ;If Multiplier bit is '0', skip next
        call    addw            ;BCCH,BCCL = BCCH,BCCL + ACCH,ACCL
        clrc                    ;Clear Carry bit
        rlf     accl,f          ;ACCL * 2
        rlf     acch,f          ;ACCH * 2
        djnz    bccm,mulib1     ;If not 7 times, jump to mulib1
        return
;
;
;********************************************************************
; Division [ ACCH = ACCL / W ]
;    Input parameter   accl = Dividend
;                      W = Divisor
;    Output parameter  acch = Quotient
;                      accl = Rest
;    Used register     acch,accl,bcch,bccl
;********************************************************************
divb:
        clrf    bccl            ;Clear BCCL
        movwf   bcch            ;Save Divisor to BCCH
        movf    bcch,f          ;Divisor = 0 ?
        jpnz    divb1           ;No. Jump to divb1
        clrf    acch            ;Yes. Error. Clear ACCH
        clrf    accl            ;Clear ACCL
        goto    divb3           ;Jump to return
divb1:
        clrc                    ;Clear Carry bit
        incf    bccl,f          ;BCCL + 1
        rlf     bcch,f          ;Find bit'1' of Divisor
        jpnc    divb1           ;If bit = 0, jump to divb1
        rrf     bcch,f          ;Return BCCH
        clrf    acch            ;Clear ACCH
divb2:
        movf    bcch,w          ;Read BCCH
        subwf   accl,f          ;ACCL(Dividend) - BCCH
        rlf     acch,f          ;Check Carry bit
        btfss   acch,0          ;ACCL(Dividend) < BCCH ?
        addwf   accl,f          ;Yes. ACCL(Dividend) + BCCH
        clrc                    ;No. Clear Carry bit
        rrf     bcch,f          ;BCCH / 2
        djnz    bccl,divb2      ;If BCCL - 1 != 0, jump to divb2
divb3:
        return
;
;
;********************************************************************
; Subtraction [ ACCH,ACCL = ACCH,ACCL - BCCH,BCCL ( No borrows ) ]
;    Input parameter   acch,accl = Minuend
;                      bcch,bccl = Subtrahend
;    Output parameter  acch,accl = Subtraction result
;                      acc_flg = 1 : Subtraction success
;                      acc_flg = 0 : Subtraction failure
;    Used register     acch,accl,bcch,bccl,acc_flg
;********************************************************************
us_subw:
        movf    acch,w          ;Read ACCH
        movwf   acc_flg         ;Save ACCH
        movf    bcch,w          ;Read BCCH
        subwf   acch,f          ;ACCH = ACCH - BCCH
        jpnb    us_subw1        ;If ACCH > BCCH, jump to us_subw1
        addwf   acch,f          ;If no, return ACCH
        goto    us_subw2        ;Jump to Error
us_subw1:
        movf    bccl,w          ;Read BCCL
        subwf   accl,f          ;ACCL = ACCL - BCCL
        jpnb    us_subw3        ;If ACCL > BCCL, jump to us_subw3
        movf    acch,f          ;Read ACCH
        jpz     us_subw10       ;If ACCH = 0, jump to Error
        decf    acch,f          ;If ACCH > 0, ACCH - 1
        goto    us_subw3        ;Jump to Success
us_subw10:
        addwf   accl,f          ;Return ACCL
        movf    acc_flg,w       ;Read ACCH
        movwf   acch            ;Return ACCH
us_subw2:
        clrf    acc_flg         ;Set failure flag
        goto    us_subw4        ;Jump to return
us_subw3:
        movlw   1               ;Set flag data
        movwf   acc_flg         ;Set success flag
us_subw4:
        return
;
;
;********************************************************************
; Initialization of date and time
;    Input parameter   None
;    Output parameter  tm_1s,tm_10s,tm_1m,tm_10m,tm_1h,tm_10h,
;                      tm_1d,tm_10d,tm_1mth,tm_10mth,tm_wd,
;                      tm_1y,tm_10y,tm_ndh,tm_ndl
;    Used register     None
;********************************************************************
init_time:
        clrf    tm_1s           ;Clear 1st of second counter
        clrf    tm_10s          ;Clear 10th of second counter
        clrf    tm_1m           ;Clear 1st of minute counter
        clrf    tm_10m          ;Clear 10th of minute counter
        clrf    tm_1h           ;Clear 1st of hour counter
        clrf    tm_10h          ;Clear 10th of hour counter
        clrf    tm_1d           ;Clear 1st of day counter
        incf    tm_1d,f         ;Set 1st of day
        clrf    tm_10d          ;Clear 10th of day counter
        clrf    tm_1mth         ;Clear 1st of mounth counter
        incf    tm_1mth,f       ;Set January
        clrf    tm_10mth        ;Clear 10th of mounth counter
        clrf    tm_1y           ;Clear 1st of year counter
        clrf    tm_10y          ;Clear 10th of year counter
        clrf    tm_wd           ;Clear week counter
        clrf    tm_ndh          ;Clear higher ND counter
        clrf    tm_ndl          ;Clear lower ND counter
        incf    tm_ndl,f        ;Set 1st to ND
        clrf    tm_uy           ;Clear leap year flag counter
        incf    tm_uy,f         ;Set leap year flag
        return
;
;
;********************************************************************
; Initialization of decoded date and time
;    Input parameter   None
;    Output parameter  dctm_1m,dctm_10m,dctm_1h,dctm_10h,
;                      dctm_1y,dctm_10y,dctm_wd
;                      dctm_1nd,dctm_10nd,dctm_100nd
;    Used register     None
;********************************************************************
init_dec_time:
        clrf    dctm_1m         ;Clear 1st of decoded minute
        clrf    dctm_10m        ;Clear 10th of decoded minute
        clrf    dctm_1h         ;Clear 1st of decoded hour
        clrf    dctm_10h        ;Clear 10th of decoded hour
        clrf    dctm_1y         ;Clear 1st of decoded year
        clrf    dctm_10y        ;Clear 10th of decoded year
        clrf    dctm_wd         ;Clear decoded week
        clrf    dctm_1nd        ;Clear 1st of decoded ND
        clrf    dctm_10nd       ;Clear 10th of decoded ND
        clrf    dctm_100nd      ;Clear 100th of decoded ND
	return
;
;
;********************************************************************
; Initialization of decoded backup date and time
;    Input parameter   None
;    Output parameter  bk_dctm_1m,bk_dctm_10m,bk_dctm_1h,bk_dctm_10h,
;                      bk_dctm_1y,bk_dctm_10y,
;                      bk_dctm_wd,bk_dctm_1nd,bk_dctm_10nd,
;                      bk_dctm_100nd
;    Used register     None
;********************************************************************
init_bk_dec_time:
        clrf    bk_dctm_1m      ;Clear 1st of decoded backup minute
        clrf    bk_dctm_10m     ;Clear 10th of decoded backup minute
        clrf    bk_dctm_1h      ;Clear 1st of decoded backup hour
        clrf    bk_dctm_10h     ;Clear 10th of decoded backup hour
        clrf    bk_dctm_1y      ;Clear 1st of decoded backup year
        clrf    bk_dctm_10y     ;Clear 10th of decoded backup year
        clrf    bk_dctm_wd      ;Clear decoded backup week
        clrf    bk_dctm_1nd     ;Clear 1st of decoded backup ND
        clrf    bk_dctm_10nd    ;Clear 10th of decoded backup ND
        clrf    bk_dctm_100nd   ;Clear 100th of decoded backup ND
        return
;
;
;********************************************************************
; Initialization of decode check counter
;    Input parameter   None
;    Output parameter  tm_chk_cnt_m,tm_chk_cnt_h,tm_chk_cnt_nd,
;                      tm_chk_cnt_wd,tm_chk_cnt_y
;    Used register     None
;********************************************************************
init_tm_chk_cnt:
        clrf    tm_chk_cnt_m    ;Clear time check counter (Munite)
        clrf    tm_chk_cnt_h    ;Clear time check counter (Hour)
        clrf    tm_chk_cnt_nd   ;Clear time check counter (Day)
        clrf    tm_chk_cnt_wd   ;Clear time check counter (Week)
        clrf    tm_chk_cnt_y    ;Clear time check counter (Year)
        return
;
;
;********************************************************************
; Check decode check counter
;    Input parameter   tm_chk_cnt_m,tm_chk_cnt_h,tm_chk_cnt_nd,
;                      tm_chk_cnt_wd,tm_chk_cnt_y
;    Output parameter  freg0 = 0 : Decoded
;                      freg0 = 1 : Decode un-completing
;    Used register     None
;********************************************************************
chk_tm_chk_cnt:
        clrf    freg0                  ;Clear freg0
        cp      tm_chk_cnt_m,chk_val   ;Check minute
        jpb     chk_tm_chk_cnt1        ;If m < val, jump cnt1
        cp      tm_chk_cnt_h,chk_val   ;Check hour
        jpb     chk_tm_chk_cnt1        ;If h < val, jump cnt1
        cp      tm_chk_cnt_nd,chk_val  ;Check normalized day
        jpb     chk_tm_chk_cnt1        ;If nd < val, jump cnt1
        cp      tm_chk_cnt_wd,chk_val  ;Cjeck week
        jpb     chk_tm_chk_cnt1        ;If w < val, jump cnt1
        cp      tm_chk_cnt_y,chk_val   ;Check year
        jpb     chk_tm_chk_cnt1        ;If y < val, jump cnt1
        goto    chk_tm_chk_cnt2        ;All OK, then jump return
chk_tm_chk_cnt1:
        incf    freg0,f                ;Set freg0 = 1
chk_tm_chk_cnt2:
        return
;
;
;********************************************************************
; Adjustment in 0 seconds
;    Input parameter   tm_1s,tm_10s
;    Output parameter  tm_1s,tm_10s,tm_1m,tm_10m,tm_1h,tm_10h,
;                      tm_1d,tm_10d,tm_1mth,tm_10mth,tm_wd,
;                      tm_1y,tm_10y,tm_ndh,tm_ndl
;    Used register     None
;********************************************************************
adjust_0s:
        clrf    unadj_cnt       ;Clear time non-correction counter
        cp      tm_10s,3        ;Present second >= 30 seconds ?
        jpnb    adjust_0s1      ;Yes. Jump to 59 seconds setting
        clrf    tm_1s           ;No. Clear 1st of second
        clrf    tm_10s          ;Clear 10th of second
        goto    adjust_0s2      ;Jump to return
adjust_0s1:
        movlw   9
        movwf   tm_1s           ;Set 1st of second to 9
        movlw   5
        movwf   tm_10s          ;Set 10th of second to 5
        call    inc_time        ;1 second + 1
adjust_0s2:
        return
;
;
;********************************************************************
; Copy the decoded time to display time
;    Input parameter   bk_dctm_1m,bk_dctm_10m,bk_dctm_1h,bk_dctm_10h,
;                      bk_dctm_1nd,bk_dctm_10nd,bk_dctm_100nd,
;                      bk_dctm_1y,bk_dctm_10y,bk_dctm_wd
;    Output parameter  tm_1m,tm_10m,tm_1h,tm_10h,
;                      tm_1d,tm_10d,tm_1mth,tm_10mth,tm_wd,
;                      tm_1y,tm_10y,tm_ndh,tm_ndl,tm_uy
;    Used register     acch,accm,accl,bcch,bccm,bccl
;********************************************************************
copy_time:
        movf    bk_dctm_1m,w    ;Read 1st of decoded backup minute
        movwf   tm_1m           ;Set 1st of minute counter
        movf    bk_dctm_10m,w   ;Read 10th of decoded backup minute
        movwf   tm_10m          ;Set 10th of minute
        movf    bk_dctm_1h,w    ;Read 1st of decoded backup hour
        movwf   tm_1h           ;Set 1st of hour counter
        movf    bk_dctm_10h,w   ;Read 10th of decoded backup hour
        movwf   tm_10h          ;Set 10th of hour counter
        movf    bk_dctm_1y,w    ;Read 1st of decoded backup year
        movwf   tm_1y           ;Set 1st of year counter
        movf    bk_dctm_10y,w   ;Read 10th of decoded backup year
        movwf   tm_10y          ;Set 10th of year counter
        movf    bk_dctm_wd,w    ;Read decoded backup week
        movwf   tm_wd           ;Set week counter
        clrf    bcch            ;Clear BCCH
        movf    bk_dctm_1nd,w   ;Read 1st of decoded backup ND
        movwf   bccl            ;Set 1st of decoded backup ND
        movf    bk_dctm_10nd,w  ;Read 10th of decoded backup ND
        movwf   accl            ;Set 10th of decoded backup ND
        movlw   10              ;Set Multiplier
        call    mulib           ;BCCH,BCCL = BCCH,BCCL + (ACCL * W)
        movf    bk_dctm_100nd,w ;Read 100th of decoded backup ND
        movwf   accl            ;Set 100th of decoded backup ND
        movlw   100             ;Set Multiplier
        call    mulib           ;BCCH,BCCL = BCCH,BCCL + (ACCL * W)
        movf    bcch,w          ;Read decoded backup higher ND
        movwf   tm_ndh          ;Set higher ND counter
        movf    bccl,w          ;Read decoded backup lower ND
        movwf   tm_ndl          ;Set lower ND counter
        clrf    tm_uy           ;Clear leap year flag
        movlw   b'00000001'     ;Set mask
        andwf   tm_10y,w        ;Pickup LSB of 10th of year
        jpz     copy_time1      ;If LSB = 0, jump to copy_time1
        movlw   2               ;If NO, set 2 into W
copy_time1:
        addwf	tm_1y,w         ;Add index
        andlw	b'00000011'     ;Set mask
        skpnz                   ;Leap year ?
        incf	tm_uy,f         ;Yes. Set leap year flag
        call	cvt_nd2day      ;Convert ND to month and day data
        return
;
;
;********************************************************************
; Time decoding
;    Input parameter   None
;    Output parameter  dec_sts2 = 0 : Decoded normally
;                      dec_sts2 != 0 : Failure
;    Used register     freg0
;********************************************************************
dec_time:
        btfss   det_mark_1st      ;First 0 sec marker was detected ?
        goto    dec_time00        ;No. Jump to dec_time00
        btfsc   time_adjust       ;Time adjust flag was set ?
        goto    dec_time0         ;Yes. Jump to dec_time0
        btfsc   det_marker        ;No. 0 sec marker detected flag was set ?
        goto    dec_time1         ; Yes. Jump to dec_time1
dec_time00:
        bsf     dec_tco_err       ; No. Set TCO error flag
        btfsc   sync_sts          ;Synchronized ?
        call    search_mark       ;Yes. Search marker
        goto    dec_time_exit     ;No. Jump to return
dec_time0:
        btfsc   det_marker        ;0 sec marker detected flag was set ?
        goto    dec_time1         ;Yes. Jump to dec_time1
        btfsc   sync_sts          ;No. Synchronized ?
        call    search_mark       ; Yes. Search marker
        goto    dec_time_exit     ; No. Jump to return
;
dec_time1:                        ;Time decode process
        btfsc   tco_valid,tco_err ;Time code error ?
        goto    dec_time12        ;Yes. Jump to dec_time12
        btfss   tco_valid,tco_vp  ;No. TCO period valid = 1 ?
        goto    dec_time12        ; No. Jump to dec_time12
        btfss   tco_valid,tco_vc  ; Yes. Time code valid = 1 ?
        goto    dec_time12        ;  No. Jump to dec_time12
        bcf     dec_tco_err       ;  Yes. Clear TCO error flag
        cp      tco_code,tco_m    ;Code is marker ?
        jpnz    dec_time2         ;Yes. Jump to dec_time2
        goto    dec_time72        ;No. Jump to dec_time72
dec_time12:
        bsf     dec_tco_err       ;Set TCO error flag
        goto    dec_time72        ;Jump to dec_time72
;
dec_time2:                        ;Decode minute
        cp      tm_10s,1          ;After 10 seconds ?
        jpnb    dec_time3         ;Yes. Jump to dec_time3(Hour)
        call    dec_time_m        ;No. Decode minute
        goto    dec_time_exit     ;Jump to return
;
dec_time3:                        ;Decode hour
        cp      tm_10s,2          ;After 20 seconds ?
        jpnb    dec_time4         ;Yes. Jump to dec_time4(ND)
        call    dec_time_h        ;No. Decode hour
        goto    dec_time_exit     ;Jump to return
;
dec_time4:                        ;Decode ND
        cp      tm_10s,4          ;After 40 seconds ?
        jpnb    dec_time5         ;Yes. Jump to dec_time5(Year)
        call    dec_time_nd       ;No. Decode ND
        goto    dec_time_exit     ;Jump to return
;
dec_time5:                        ;Decode year
        cp      bk_dctm_1m,5      ;After 5 minutes ?
        jpnz    dec_time52        ;No. Jump to dec_time52
        cp      bk_dctm_10m,1     ;Yes. After 10 minutes ?
        jpz     dec_time51        ; Yes. Jump to dec_time51
        cp      bk_dctm_10m,4     ; No. After 40 minutes ?
        jpnz    dec_time52        ;  No. Jump to dec_time52
dec_time51:                       ;Set error flag when 15 and 45 minutes
        bsf     dec_year_err      ;Set year decode error falg
        goto    dec_time72        ;Jump to dec_time72
dec_time52:
        cp      tm_10s,5          ;After 50 seconds ?
        jpnb    dec_time6         ;Yes. Jump to dec_time6(week)
        cp      tm_1s,2           ;No. Before 40 seconds ?
        jpb     dec_time_exit     ;Jump to return
        call    dec_time_y        ;Decode year
        goto    dec_time_exit     ;Jump to return
;
dec_time6:                        ;Decode week
        cp      tm_1s,4           ;After 53 seconds ?
        jpnb    dec_time7
        call    dec_time_wd       ;Decode week
        goto    dec_time_exit     ;Jump to return
;
dec_time7:                        ;Decode leap year
        cp      tm_1s,6           ;After 55 seconds ?
        jpnb    dec_time72
        goto    dec_time_exit     ;Jump to return
dec_time72:
        cp      tm_1s,6           ;55 seconds ?
        jpnz    dec_time_exit
        cp      tm_10s,5
        jpnz    dec_time_exit
        call    init_dec_time     ;Initialization of decoded date and time
        btfsc   dec_time_sts      ;Time decoded ?
        goto    dec_time_exit     ;No. Jump to return
        call    chk_tm_chk_cnt    ;Check decode check counter
        movf    freg0,f           ;All data decoded ?
        jpnz    dec_time73        ;No.
        call    copy_time         ;Yes. Copy the decoded time to display time
        bsf     dec_time_1st      ;Set first time decode flag
        bsf     dec_time_sts      ;Set time decoded status
        goto    dec_time_exit     ;Jump to return
dec_time73:
        call    inc_dec_time      ;Decode backup time is set forward for 1 minute.
        clrf    dec_sts2          ;Clear decode status 2
;
dec_time_exit
        return
;
;
;********************************************************************
; O seconds marker detection
;    Input parameter   None
;    Output parameter  det_marker = 0 : Failure
;                      det_marker = 1 : Decoded normally
;    Used register     dec_sts2,mark_pd_cnt,zero_cnt
;********************************************************************
search_mark:
        clrf    dec_sts2          ;Clear decode status 2
        incf    mark_pd_cnt,f     ;0 seconds marker period + 1
        btfsc   tco_valid,tco_err ;Time code error ?
        goto    search_mark5      ;Yes. Jump to Error processing
        btfss   tco_valid,tco_vp  ;TCO period valid ?
        goto    search_mark5      ;No. Jump to Error processing
        cp      tco_code,tco_1    ;One decode ?
        jpz     search_mark4
        cp      tco_code,tco_m    ;Marker code ?
        jpz     search_mark1      ;Yes. Jump to marker count + 1
        incf    zero_cnt,f        ;0 code counter + 1
        clrf    mark_cnt          ;Clear marker code counter
        goto    search_mark_exit
search_mark1:
        incf    mark_cnt,f        ;Marker count + 1
        cp      mark_cnt,2        ;Marker continues twice ?
        jpz     search_mark2
        cp      zero_cnt,4        ;0 code counter continues four times ?
        jpb     search_mark4
        clrf    zero_cnt
        goto    search_mark_exit
search_mark2:
        cp      mark_pd_cnt,60    ;Period of 0 seconds marker is 60 seconds ?
        jpnz    search_mark3      ;No.
        btfss   time_adjust       ;Time decode flag set ?
        goto    search_mark6
        call    adjust_0s         ;Adjust 0 seconds
        goto    search_mark7
search_mark6:
        call    init_time         ;Detect 0 seconds marker
        bsf     det_mark_1st      ;Set first 0 seconds marker detection flag
search_mark7:
        call    inc_time          ;Forward for 1 second
        call    init_bk_dec_time  ;Initialization of decode backup time
        call    init_dec_time     ;Initialization of decode time
        call    init_tm_chk_cnt   ;Clear decode time check counter
        bsf     det_marker        ;Set 0 seconds marker detection flag
        goto    search_mark_exit
search_mark3:
        clrf    mark_pd_cnt       ;Clear 0 seconds marker period
search_mark4:
        clrf    mark_cnt          ;Clear marker count
        clrf    zero_cnt          ;Clear 0 code counter
        goto    search_mark_exit
search_mark5:
        clrf    mark_cnt          ;Clear marker count
        bsf     dec_tco_err       ;Set TCO error flag
search_mark_exit:
        return
;
;
;********************************************************************
; Minute decoding
;    Input parameter   None
;    Output parameter  dec_minite_err = 0 : Decoded normally
;                      dec_minute_err = 1 : Failure
;                      dctm_1m,dctm_10m   : Decoded result
;    Used register     tm_1s,tm_10s,dctm_1m,dctm_10m
;********************************************************************
dec_time_m:
        btfss   dec_minute_err    ;Minute decoding error ?
        goto    dec_time_m1       ;No.
        goto    dec_time_m6       ;Yes. Jump to return
dec_time_m1:
        cp      tm_1s,5           ;After 4 seconds ?
        jpnb    dec_time_m2       ;Yes. Jump to 1st of minute decoding
        rrf     tco_code,w        ;No. Set time code to carry flag
        rlf     dctm_10m,f        ;10th of minute decoding
        cp      tm_1s,4           ;End of 10th of minute ?
        jpb     dec_time_m6       ;No. Jump to return
        cp      dctm_10m,6        ;Yes. Check 10th of minute value
        jpb     dec_time_m6       ;If out of range, jump to return
        bsf     dec_minute_err    ;Set error flag of minute decoding
        goto    dec_time_m6       ;Jump to return
dec_time_m2:                      ;1st of minute decoding
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_1m,f         ;1st of minute decoding
        cp      tm_1s,9           ;End of 1st of minute ?
        jpb     dec_time_m6       ;No. Jump to return
        cp      dctm_1m,10        ;Yes. Check 1st of minute value
        jpb     dec_time_m3       ;If out of range, jump to dec_time_m3
        bsf     dec_minute_err    ;Set error flag of minute decoding
        goto    dec_time_m6       ;Jump to return
dec_time_m3:
        movf    tm_chk_cnt_m,f    ;Minute decode check counter = 0 ?
        jpnz    dec_time_m4       ;No.
        movf    dctm_1m,w         ;Yes. Read decoded 1st of minute
        movwf   bk_dctm_1m        ;Set decoded 1st of minute to backup
        movf    dctm_10m,w        ;Read decoded 10th of minute
        movwf   bk_dctm_10m       ;Set decoded 10th of minute to backup
        incf    tm_chk_cnt_m,f    ;Minute decode check counter + 1
        goto    dec_time_m6       ;Jump to return
dec_time_m4:
        movf    dctm_1m,w         ;Read decoded 1st of minute
        cpw     bk_dctm_1m        ;Decoded 1st of minute = backup ?
        jpnz    dec_time_m5       ;No.
        movf    dctm_10m,w        ;Yes. Read decoded 10th of minute
        cpw     bk_dctm_10m       ;Decoded 10th of minute = backup ?
        jpnz    dec_time_m5       ;No.
        incf    tm_chk_cnt_m,f    ;Minute decode check counter + 1
        cp      tm_chk_cnt_m,chk_val+1 ;Check counter value(3+1)
        jpb     dec_time_m6       ;If not over, jump to return
        decf    tm_chk_cnt_m,f    ;Minute decode check counter - 1
        goto    dec_time_m6       ;Jump to return
dec_time_m5:
        movf    dctm_1m,w         ;Read decoded 1st of minute
        movwf   bk_dctm_1m        ;Set decoded 1st of minute to backup
        movf    dctm_10m,w        ;Read decoded 10th of minute
        movwf   bk_dctm_10m       ;Set decoded 10th of minute to backup
        clrf    tm_chk_cnt_m      ;Clear Minute decode check counter
        incf    tm_chk_cnt_m,f    ;Minute decode check counter + 1
dec_time_m6:
        return
;
;
;********************************************************************
; Hour decoding
;    Input parameter   None
;    Output parameter  dec_hour_err = 0   : Decoded normally
;                      dec_hour_err = 1   : Failure
;                      dctm_1h,dctm_10h   : Decoded result
;    Used register     tm_1s,tm_10s,dctm_1h,dctm_10h
;********************************************************************
dec_time_h:
        btfss   dec_hour_err      ;Hour decoding error ?
        goto    dec_time_h1       ;No.
        goto    dec_time_h6       ;Yes. Jump to return
dec_time_h1:
        cp      tm_1s,5           ;After 14 seconds ?
        jpnb    dec_time_h2       ;Yes. Jump to 1st of hour decoding
        rrf     tco_code,w        ;No. Set time code to carry flag
        rlf     dctm_10h,f        ;10th of hour decoding
        cp      tm_1s,4           ;End of 10th of hour ?
        jpb     dec_time_h6       ;No. Jump to return
        cp      dctm_10h,3        ;Yes. Check 10th of hour value
        jpb     dec_time_h6       ;If out of range, jump to return
        bsf     dec_hour_err      ;Set error flag of hour decoding
        goto    dec_time_h6       ;Jump to return
dec_time_h2:                      ;1st of hour decoding
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_1h,f         ;1st of hour decoding
        cp      tm_1s,9           ;End of 1st of hour ?
        jpb     dec_time_h6       ;No. Jump to return
        cp      dctm_1h,10        ;Yes. Check 1st of hour value
        jpb     dec_time_h3       ;If out of range, jump to dec_time_h3
        bsf     dec_hour_err      ;Set error flag of hour decoding
        goto    dec_time_h6       ;Jump to return
dec_time_h3:
        movf    tm_chk_cnt_h,f    ;Hour decode check counter = 0 ?
        jpnz    dec_time_h4       ;No.
        movf    dctm_1h,w         ;Yes. Read decoded 1st of hour
        movwf   bk_dctm_1h        ;Set decoded 1st of hour to backup
        movf    dctm_10h,w        ;Read decoded 10th of hour
        movwf   bk_dctm_10h       ;Set decoded 10th of hour to backup
        incf    tm_chk_cnt_h,f    ;Hour decode check counter + 1
        goto    dec_time_h6       ;Jump to return
dec_time_h4:
        movf    dctm_1h,w         ;Read decoded 1st of hour
        cpw     bk_dctm_1h        ;Decoded 1st of hour = backup ?
        jpnz    dec_time_h5       ;No.
        movf    dctm_10h,w        ;Yes. Read decoded 10th of hour
        cpw     bk_dctm_10h       ;Decoded 10th of hour = backup ?
        jpnz    dec_time_h5       ;No.
        incf    tm_chk_cnt_h,f    ;Hour decode check counter + 1
        cp      tm_chk_cnt_h,chk_val+1 ;Check counter value(3+1)
        jpb     dec_time_h6       ;If not over, jump to return
        decf    tm_chk_cnt_h,f    ;Hour decode check counter - 1
        goto    dec_time_h6       ;Jump to return
dec_time_h5:
        movf    dctm_1h,w         ;Read decoded 1st of hour
        movwf   bk_dctm_1h        ;Set decoded 1st of hour to backup
        movf    dctm_10h,w        ;Read decoded 10th of hour
        movwf   bk_dctm_10h       ;Set decoded 10th of hour to backup
        clrf    tm_chk_cnt_h      ;Clear Hour decode check counter
        incf    tm_chk_cnt_h,f    ;Hour decode check counter + 1
dec_time_h6:
        return
;
;
;********************************************************************
; Normalized day(ND) decoding
;    Input parameter   None
;    Output parameter  dec_nday_err = 0   : Decoded normally
;                      dec_nday_err = 1   : Failure
;                      dctm_1nd,dctm_10nd,dctm_100nd : Decoded result
;    Used register     tm_1s,tm_10s,dctm_1nd,dctm_10nd,dctm_100nd
;********************************************************************
dec_time_nd:
        btfss   dec_nday_err      ;ND decoding error ?
        goto    dec_time_nd1      ;No.
        goto    dec_time_nd7      ;Yes. Jump to return
dec_time_nd1:
        cp      tm_10s,3          ;After 30 seconds ?
        jpnb    dec_time_nd3      ;Yes. Jump to 1st of ND decoding
        cp      tm_1s,5           ;After 24 seconds ?
        jpnb    dec_time_nd2      ;Yes. Jump to 10th of ND decoding
        rrf     tco_code,w        ;No. Set time code to carry flag
        rlf     dctm_100nd,f      ;100th of ND decoding
        cp      tm_1s,4           ;End of 100th of ND ?
        jpb     dec_time_nd7      ;No. Jump to return
        cp      dctm_100nd,4      ;Yes. Check 100th of ND value
        jpb     dec_time_nd7      ;If out of range, jump to return
        bsf     dec_nday_err      ;Set error flag of ND decoding
        goto    dec_time_nd7      ;Jump to return
dec_time_nd2:                     ;10th of ND decoding
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_10nd,f       ;10th of ND decoding
        cp      tm_1s,9           ;End of 10th of ND ?
        jpb     dec_time_nd7      ;No. Jump to return
        cp      dctm_10nd,10      ;Yes. Check 10th of ND value
        jpb     dec_time_nd7      ;If out of range, jump to retun ??
        bsf     dec_nday_err      ;Set error flag of ND decoding
        goto    dec_time_nd7      ;Jump to return
dec_time_nd3:                     ;1st of ND decoding
        cp      tm_1s,5           ;After 34 seconds ?
        jpnb    dec_time_nd7      ;Yes. Jump to return
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_1nd,f        ;1st of ND decoding
        cp      tm_1s,4           ;End of 1st of ND ?
        jpb     dec_time_nd7      ;No. Jump to return
        cp      dctm_1nd,10       ;Yes. Check 1st of ND value
        jpb     dec_time_nd4      ;If out of range, jump to dec_time_nd4
        bsf     dec_nday_err      ;Set error flag of ND decoding
        goto    dec_time_nd7      ;Jump to return
dec_time_nd4:
        movf    tm_chk_cnt_nd,f   ;ND decode check counter = 0 ?
        jpnz    dec_time_nd5      ;No.
        movf    dctm_1nd,w        ;Yes. Read decoded 1st of ND
        movwf   bk_dctm_1nd       ;Set decoded 1st of ND to backup
        movf    dctm_10nd,w       ;Read decoded 10th of ND
        movwf   bk_dctm_10nd      ;Set decoded 10th of ND to backup
        movf    dctm_100nd,w      ;Read decoded 100th of ND
        movwf   bk_dctm_100nd     ;Set decoded 100th of ND to backup
        incf    tm_chk_cnt_nd,f   ;ND decode check counter + 1
        goto    dec_time_nd7      ;Jump to return
dec_time_nd5:
        movf    dctm_1nd,w        ;Read decoded 1st of ND
        cpw     bk_dctm_1nd       ;Decoded 1st of ND = backup ?
        jpnz    dec_time_nd6      ;No.
        movf    dctm_10nd,w       ;Yes. Read decoded 10th of ND
        cpw     bk_dctm_10nd      ;Decoded 10th of ND = backup ?
        jpnz    dec_time_nd6      ;No.
        movf    dctm_100nd,w      ;Yes. Read decoded 100th of ND
        cpw     bk_dctm_100nd     ;Decoded 100th of ND = backup ?
        jpnz    dec_time_nd6      ;No.
        incf    tm_chk_cnt_nd,f   ;ND decode check counter + 1
        cp      tm_chk_cnt_nd,chk_val+1 ;Check counter value(3+1)
        jpb     dec_time_nd7      ;If not over, jump to return
        decf    tm_chk_cnt_nd,f   ;ND decode check counter - 1
        goto    dec_time_nd7      ;Jump to return
dec_time_nd6:
        movf    dctm_1nd,w        ;Read decoded 1st of ND
        movwf   bk_dctm_1nd       ;Set decoded 1st of ND to backup
        movf    dctm_10nd,w       ;Read decoded 10th of ND
        movwf   bk_dctm_10nd      ;Set decoded 10th of ND to backup
        movf    dctm_100nd,w      ;Read decoded 100th of ND
        movwf   bk_dctm_100nd     ;Set decoded 100th of ND to backup
        clrf    tm_chk_cnt_nd     ;Clear ND decode check counter
        incf    tm_chk_cnt_nd,f   ;ND decode check counter + 1
dec_time_nd7:
        return
;
;
;********************************************************************
; Year decoding
;    Input parameter   None
;    Output parameter  dec_year_err = 0   : Decoded normally
;                      dec_year_err = 1   : Failure
;                      dctm_1y,dctm_10y   : Decoded result
;    Used register     tm_1s,tm_10s,dctm_1y,dctm_10y
;********************************************************************
dec_time_y:
        btfss   dec_year_err      ;Hour decoding error ?
        goto    dec_time_y1       ;No.
        goto    dec_time_y6       ;Yes. Jump to return
dec_time_y1:
        cp      tm_1s,6           ;After 45 seconds ?
        jpnb    dec_time_y2       ;Yes. Jump to 1st of year decoding
        rrf     tco_code,w        ;No. Set time code to carry flag
        rlf     dctm_10y,f        ;10th of year decoding
        cp      tm_1s,4           ;End of 10th of year ?
        jpb     dec_time_y6       ;No. Jump to return
        cp      dctm_10y,10       ;Yes. Check 10th of year value
        jpb     dec_time_y6       ;If out of range, jump to return
        bsf     dec_year_err      ;Set error flag of year decoding
        goto    dec_time_y6       ;Jump to return
dec_time_y2:                      ;1st of year decoding
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_1y,f         ;1st of year decoding
        cp      tm_1s,9           ;End of 1st of year ?
        jpb     dec_time_y6       ;No. Jump to return
        cp      dctm_1y,10        ;Yes. Check 1st of year value
        jpb     dec_time_y3       ;If out of range, jump to dec_time_y3
        bsf     dec_year_err      ;Set error flag of year decoding
        goto    dec_time_y6       ;Jump to return
dec_time_y3:
        movf    tm_chk_cnt_y,f    ;Year decode check counter = 0 ?
        jpnz    dec_time_y4       ;No.
        movf    dctm_1y,w         ;Yes. Read decoded 1st of year
        movwf   bk_dctm_1y        ;Set decoded 1st of year to backup
        movf    dctm_10y,w        ;Read decoded 10th of year
        movwf   bk_dctm_10y       ;Set decoded 10th of year to backup
        incf    tm_chk_cnt_y,f    ;Year decode check counter + 1
        goto    dec_time_y6       ;Jump to return
dec_time_y4:
        movf    dctm_1y,w         ;Read decoded 1st of year
        cpw     bk_dctm_1y        ;Decoded 1st of year = backup ?
        jpnz    dec_time_y5       ;No.
        movf    dctm_10y,w        ;Yes. Read decoded 10th of year
        cpw     bk_dctm_10y       ;Decoded 10th of year = backup ?
        jpnz    dec_time_y5       ;No.
        incf    tm_chk_cnt_y,f    ;Year decode check counter + 1
        cp      tm_chk_cnt_y,chk_val+1 ;Check counter value(3+1)
        jpb     dec_time_y6       ;If not over, jump to return
        decf    tm_chk_cnt_y,f    ;Year decode check counter - 1
        goto    dec_time_y6       ;Jump to return
dec_time_y5:
        movf    dctm_1y,w         ;Read decoded 1st of year
        movwf   bk_dctm_1y        ;Set decoded 1st of year to backup
        movf    dctm_10y,w        ;Read decoded 10th of year
        movwf   bk_dctm_10y       ;Set decoded 10th of year to backup
        clrf    tm_chk_cnt_y      ;Clear Year decode check counter
        incf    tm_chk_cnt_y,f    ;Year decode check counter + 1
dec_time_y6:
        return
;
;
;********************************************************************
; Week decoding
;    Input parameter   None
;    Output parameter  dec_wday_err = 0   : Decoded normally
;                      dec_wday_err = 1   : Failure
;                      dctm_wd            : Decoded result
;    Used register     tm_1s,tm_10s,dctm_wd
;********************************************************************
dec_time_wd:
        btfss   dec_wday_err      ;Week decoding error ?
        goto    dec_time_wd1      ;No.
        goto    dec_time_wd5      ;Yes. Jump to return
dec_time_wd1:
        rrf     tco_code,w        ;Set time code to carry flag
        rlf     dctm_wd,f         ;Week decoding
        cp      tm_1s,3           ;End of week decoding ?
        jpb     dec_time_wd5      ;No. Jump to return
        cp      dctm_wd,7         ;Yes. Check week value
        jpb     dec_time_wd2      ;If out of range, jump to dec_time_wd2
        bsf     dec_wday_err      ;Set error flag of week decoding
        goto    dec_time_wd5      ;Jump to return
dec_time_wd2:
        movf    tm_chk_cnt_wd,f   ;Week decode check counter = 0 ?
        jpnz    dec_time_wd3      ;No.
        movf    dctm_wd,w         ;Yes. Read decoded week
        movwf   bk_dctm_wd        ;Set decoded week to backup
        incf    tm_chk_cnt_wd,f   ;Week decode check counter + 1
        goto    dec_time_wd5      ;Jump to return
dec_time_wd3:
        movf    dctm_wd,w         ;Read decoded week
        cpw     bk_dctm_wd        ;Decoded week = backup ?
        jpnz    dec_time_wd4      ;No.
        incf    tm_chk_cnt_wd,f   ;Week decode check counter + 1
        cp      tm_chk_cnt_wd,chk_val+1 ;Check counter value(3+1)
        jpb     dec_time_wd5      ;If not over, jump to return
        decf    tm_chk_cnt_wd,f   ;Week decode check counter - 1
        goto    dec_time_wd5      ;Jump to return
dec_time_wd4:
        movf    dctm_wd,w         ;Read decoded week
        movwf   bk_dctm_wd        ;Set decoded week to backup
        clrf    tm_chk_cnt_wd     ;Clear Week decode check counter
        incf    tm_chk_cnt_wd,f   ;Week decode check counter + 1
dec_time_wd5:
        return
;
;
;********************************************************************
; 100 microseconds delay
;    Input parameter   w = Delay count
;    Output parameter  None
;    Used register     delay_cnt100u
;********************************************************************
delay100u:
        rbank1                    ;Set register bank 1
        di                        ;Disable interrupt
        movwf   delay_cnt100u     ;Set delay count
        ei                        ;Enable interrupt
delay100u1:
        movf    delay_cnt100u,f   ;Read delay count
        jpnz    delay100u1        ;If count != 0, jump to delay100u1
        rbank0                    ;Count = 0, set register bank 0
        return
;
;
;********************************************************************
; 10 miliseconds delay
;    Input parameter   w = Delay count
;    Output parameter  None
;    Used register     delay_cnt10m
;********************************************************************
delay10m:
        rbank1                    ;Set register bank 1
        di                        ;Disable interrupt
        movwf   delay_cnt10m      ;Set delay count
        ei                        ;Enable interrupt
delay10m1:
        movf    delay_cnt10m,f    ;Read delay count
        jpnz    delay10m1         ;If count != 0, jump to delay10m1
        rbank0                    ;Count = 0, set register bank 0
        return
;
;
;####################################################################
;
        org     0x800             ;Program page 1
;
;
;********************************************************************
; Serial communication processing
;    Input parameter   rx_cc       = Number of the receiving characters
;                      rx_dat(0-3) = Received data
;    Output parameter  None
;    Used register     
;********************************************************************
sio_job:
        bcf     pie1,rcie       ;Disable serial receive interruption
        movf    fsr,w
        movwf   tmp_fsr2
        call    cmd_check       ;Check serial receive command
        cp      freg0,0xff      ;Command error ?
        jpz     sio_job_err     ;Yes. Jump to error
        btfsc   stim_out        ;Under suspension ?
        goto    sio_job_err     ;Yes. Jump to error
        cp      freg0,cmdtime   ;Command = 0 ?
        jpnz    sio_job1        ;No. Next
        call    sout_time       ;Send out Time
        goto    sio_job_exit    ;Jump to exit
sio_job1:
        cp      freg0,cmddate   ;Command = 1 ?
        jpnz    sio_job2        ;No. Next
        call    sout_date       ;Send out Date
        goto    sio_job_exit    ;Jump to exit
sio_job2:
        cp      freg0,cmdstim   ;Command = 2 ?
        jpnz    sio_job3        ;No. Next
        call    sout_stime      ;Set Time synchronized with second
        goto    sio_job_exit    ;Jump to exit
sio_job3:
        cp      freg0,cmdstus   ;Command = 3 ?
        jpnz    sio_job4        ;No. Next
        call    sout_status     ;Send out time adjusted status
        goto    sio_job_exit    ;Jump to exit
sio_job4:
        cp      freg0,cmdstmr   ;Command = 4 ?
        jpnz    sio_job5        ;No. Next
        call    cmd_set_timer   ;Set Time
        movf    freg0,f         ;Parameter error ?
        jpnz    sio_job_err     ;Yes. Jump to error
        goto    sio_job_exit    ;Jump to exit
sio_job5:
        cp      freg0,cmdgtmr   ;Command = 5 ?
        jpnz    sio_job6        ;No. Next
        call    sout_timer      ;Output set time
        movf    freg0,f         ;Parameter error ?
        jpnz    sio_job_err     ;Yes. Jump to error
        goto    sio_job_exit    ;Jump to exit
sio_job6:
        cp      freg0,cmddecd   ;Command = 6 ?
        jpnz    sio_job7        ;No. Next
        call    sout_decode     ;Output decode status
        goto    sio_job_exit    ;Jump to exit
sio_job7:
        cp      freg0,cmdtmon   ;Command = 7 ?
        jpnz    sio_job8        ;No. Next
        call    cmd_timer_on    ;Timer ON
        movf    freg0,f         ;Parameter error ?
        jpnz    sio_job_err     ;Yes. Jump to error
        goto    sio_job_exit    ;Jump to exit
sio_job8:
        cp      freg0,cmdtmof   ;Command = 8 ?
        jpnz    sio_job_err     ;No. Jump to error
        call    cmd_timer_off   ;Timer OFF
        movf    freg0,f         ;Parameter error ?
        jpnz    sio_job_err     ;Yes. Jump to error
        goto    sio_job_exit    ;Jump to exit

;
sio_job_err:
        call    sout_err        ;Output error
sio_job_exit:
        movlw   rx_dat
        movwf   rx_ptr          ;Set receive data pointer
        clrf    rx_cc           ;Clear recive data counter
        movf    tmp_fsr2,w
        movwf   fsr
        bcf     rx_sts,0        ;Clear recive data enable flag
        bsf     pie1,rcie       ;Enable serial receive interruption
        return
;
;
;********************************************************************
; Command check
;    Input parameter   rx_dat(0-3) = Received data
;                      cmd_tbl     = Command characters
;    Output parameter  freg0       = Command code (0xff=Error)
;    Used register     freg0,freg1,freg2,fsr,rx_dat(0-3)
;********************************************************************
cmd_check:
        clrf    freg0           ;Clear command code
        movlw   cmd_tbl
        movwf   freg1           ;Set command character pointer
        cp      rx_cc,cmd1len   ;Group1 command ?
        jpz     cmd_check1      ;Yes.
        cp      rx_cc,cmd2len   ;Group2 command ?
        jpz     cmd_check1      ;Yes.
        cp      rx_cc,cmd3len   ;Group3 command ?
        jpnz    cmd_check4      ;No. Command Error
cmd_check1:
        movlw   rx_dat          ;Set top address of receive buffer
        movwf   fsr             ;Set receive data pointer
        clrf    freg2           ;Clear command character counter
cmd_check2:
        lcall   get_tbl_data    ;Get command character
        setpch                  ;PCLATH setting
        iorlw   0x00            ;End character ?
        jpz     cmd_check4      ;Yes. Jump to error process
        bcf     indf,5          ;Change lowercase to uppercase
        cpw     indf            ;Compare received data
        jpnz    cmd_check3      ;If not same, jump to cmd_check3
        incf    fsr,f           ;Receive data pointer + 1
        incf    freg1,f         ;Command character pointer + 1
        incf    freg2,f         ;Command character counter + 1
        cp      freg2,4         ;4 character were compared ?
        jpnz    cmd_check2      ;No. Get next character
        goto    cmd_check5      ;Yes. Jump to return
cmd_check3:
        incf    fsr,f           ;Receive data pointer + 1
        incf    freg1,f         ;Command character pointer + 1
        incf    freg2,f         ;Command character counter + 1
        cp      freg2,4         ;4 character were compared ?
        jpnz    cmd_check3      ;No. Skip until next command
        incf    freg0,f         ;Command code + 1
        goto    cmd_check1      ;Jump to next command check
cmd_check4:
        movlw   0xff
        movwf   freg0           ;Set error code
cmd_check5:
        return
;
;
;********************************************************************
; Send out Time
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_time:
        call    sout_hour       ;Send out hour
        movlw   ':'
        call    sout            ;Send out ':'
        call    sout_minute     ;Send out minute
        movlw   ':'
        call    sout            ;Send out ':'
        call    sout_second     ;Send out second
        movlw   0x0d
        call    sout            ;Send out CR
        movlw   0x0a
        call    sout            ;Send out LF
        return
;
;
;********************************************************************
; Send out Date
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_date:
        call    sout_year       ;Send out year
        movlw   '/'
        call    sout            ;Send out '/'
        call    sout_month      ;Send out month
        movlw   '/'
        call    sout            ;Send out '/'
        call    sout_day        ;Send out day
        movlw   ' '
        call    sout            ;Send out Space
        call    sout_wday       ;Send out week
        movlw   0x0d
        call    sout            ;Send out CR
        movlw   0x0a
        call    sout            ;Send out LF
        return
;
;
;********************************************************************
; Set time output suspension for synchronizing with second
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_stime:
        bsf     stim_out        ;Set time output suspension
        return
;
;
;********************************************************************
; Send out time adjusted status
;   Was an adjustment in 0 seconds done within 60 minutes?
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_status:
        cp      unadj_cnt,6     ;Unadjusted counter > 6 ?
        movlw   mes_adjust      ;Set "adjusted" message address
        jpb     sout_status1    ;No. Skip next step
        movlw   mes_unadjust    ;Yes. Set "unadjusted" message address
sout_status1:
        call    ssout           ;Send out message
        movlw   0x0d
        call    sout            ;Send out CR
        movlw   0x0a
        call    sout            ;Send out LF
        return
;
;
;********************************************************************
; Time setting command
;    Input parameter   None
;    Output parameter  freg0 = 0  : Normal parameter
;                      freg0 < 0  : Parameter error
;    Used register     freg0,freg2
;********************************************************************
cmd_set_timer:
        incf    FSR,f                ;Receive data pointer + 1
        cp      INDF,'1'             ;Check timer number
        jpb     cmd_set_timer_err    ;If < 1, Jump to error
        cp      INDF,'4'+1           ;Check timer number
        jpnb    cmd_set_timer_err    ;If >= 5. Jump to error
        movlw   '1'
        subwf   INDF,w               ;Make timer number
        movwf   freg2                ;Seve timer number
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        bcf     INDF,5               ;Change lowercase to uppercase
        clrf    tmbf_mode            ;Set timer mode to OFF
        cp      INDF,'O'             ;Check timer mode
        jpz     cmd_set_timer1       ;If OFF, jump to cmd_set_timer1
        bsf     tmbf_mode,0          ;Set timer mode to CONTINUE
        cp      INDF,'C'             ;Check timer mode
        jpz     cmd_set_timer1       ;If CONTINUE, jump to cmd_set_timer1
        incf    tmbf_mode,f          ;Set timer mode to SINGLE
        cp      INDF,'S'             ;Check timer mode
        jpnz    cmd_set_timer_err    ;If not SINGLE, jump to error
cmd_set_timer1:
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f              ;Error ?
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_str_hour        ;Set start hour
        cp      tmbf_str_hour,23+1   ;Check start hour
        jpnb    cmd_set_timer_err    ;If over 24, jump to error
;
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f              ;Error ?
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_str_minute      ;Set start minute
        cp      tmbf_str_minute,59+1 ;Check start minute
        jpnb    cmd_set_timer_err    ;If over 60, jump to error
;
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f              ;Error ?
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_str_second      ;Set start second
        cp      tmbf_str_second,59+1 ;Check start second
        jpnb    cmd_set_timer_err    ;If over 60, jump to error
;
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f              ;Error ?
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_stp_hour        ;Set stop hour
        cp      tmbf_stp_hour,23+1   ;Check stop hour
        jpnb    cmd_set_timer_err    ;If over 24, jump to error
;
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f              ;Error ?
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_stp_minute      ;Set stop minute
        cp      tmbf_stp_minute,59+1 ;Check stop minute
        jpnb    cmd_set_timer_err    ;If over 60, jump to error
;
        incf    FSR,f                ;Receive data pointer + 1
        incf    FSR,f                ;Receive data pointer + 1
        call    cv_str2num           ;Change 2nd line characters to binary
        movf    freg0,f
        jpnz    cmd_set_timer_err    ;If not 0, jump to error
        movwf   tmbf_stp_second      ;Set stop second
        cp      tmbf_stp_second,59+1 ;Check start second
        jpnb    cmd_set_timer_err    ;If over 60, jump to error
;
        movf    freg2,w              ;Read timer number
        movwf   freg0                ;Save timer number
        clrc                         ;Clear Carry bit
        rlf     freg0,f              ;Timer number * 2
        rlf     freg0,w              ;Timer number * 4
        addwf   freg0,w              ;Timer number * 6
        addwf   freg2,w              ;Timer number * 7
        addlw   ead_tm1_mode         ;Calculation of EEPROM address
        call    set_tmbf_data        ;Write timer data
        movlw   mes_timer            ;Set "Timer" message address
        call    ssout                ;Send out "Timer" message
        movlw   '1'
        addwf   freg2,w
        call    sout                 ;Send out timer number
        movlw   ' '
        call    sout                 ;Send out Space
        movlw   mes_set              ;Set message address
        call    ssout                ;Send out "set" message
        movlw   0x0d
        call    sout                 ;Send out CR
        movlw   0x0a
        call    sout                 ;Send out LF
        clrf    freg0                ;Set Normal status
        goto    cmd_set_timer_exit   ;Jump to return
cmd_set_timer_err:
        clrf    freg0
        decf    freg0,f              ;Set Error status
cmd_set_timer_exit:
        return
;
;
;********************************************************************
; Send out set time
;    Input parameter   None
;    Output parameter  freg0 = 0  : Normal parameter
;                      freg0 < 0  : Parameter error
;    Used register     freg0,freg2
;********************************************************************
sout_timer:
        clrf    freg0
        decf    freg0,f              ;Set parameter error status
        incf    FSR,f                ;Receive data pointer + 1
        cp      INDF,'1'             ;Check timer number
        jpb     sout_timer_err       ;If < 1, Jump to error
        cp      INDF,'4'+1           ;Check timer number
        jpnb    sout_timer_err       ;If >= 5. Jump to error
        movlw   '1'
        subwf   INDF,w               ;Make timer number
        movwf   freg0                ;Seve timer number
        movwf   freg2                ;Seve timer number
        clrc                         ;Clear Carry bit
        rlf     freg0,f              ;Timer number * 2
        rlf     freg0,w              ;Timer number * 4
        addwf   freg0,w              ;Timer number * 6
        addwf   freg2,w              ;Timer number * 7
        addlw   ead_tm1_mode         ;Calculation of EEPROM address
        call    get_tmbf_data        ;Read timer data
;
        movlw   mes_timer            ;Set "Timer" message address
        call    ssout                ;Send out "Timer" message
        movlw   '1'
        addwf   freg2,w
        call    sout                 ;Send out timer number
        movlw   ':'
        call    sout                 ;Send out Colon
        movlw   mes_OFF              ;Set "OFF" message address
        movf    tmbf_mode,f          ;OFF mode ?
        jpz     sout_timer1          ;Yes. Jump to sout_timer1
        movlw   mes_CONT             ;Set "CNT" message address
        btfss   tmbf_mode,0          ;If mode = CONTINUE, skip next
        movlw   mes_SINGLE           ;Set "SGL" message address
sout_timer1:
        call    ssout                ;Send out mode message
        movlw   ','
        call    sout                 ;Send out Comma
        movf    tmbf_str_hour,w
        call    sout_num2d           ;Send out strat hour
        movlw   ':'
        call    sout                 ;Send out Colon
        movf    tmbf_str_minute,w
        call    sout_num2d           ;Send out strat minute
        movlw   ':'
        call    sout                 ;Send out Colon
        movf    tmbf_str_second,w
        call    sout_num2d           ;Send out strat second
        movlw   '/'
        call    sout                 ;Send out Slash
        movf    tmbf_stp_hour,w
        call    sout_num2d           ;Send out stop hour
        movlw   ':'
        call    sout                 ;Send out Colon
        movf    tmbf_stp_minute,w
        call    sout_num2d           ;Send out stop minute
        movlw   ':'
        call    sout                 ;Send out Colon
        movf    tmbf_stp_second,w
        call    sout_num2d           ;Send out stop second
        movlw   0x0d
        call    sout                 ;Send out CR
        movlw   0x0a
        call    sout                 ;Send out LF
        clrf    freg0                ;Set Normal status
sout_timer_err:
        return
;
;
;********************************************************************
; Send out decode status
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_decode:
        movlw   mes_invalid     ;Set "invalid" message address
        btfsc   dec_time_1st    ;If 1st decode is not finished, skip next
        movlw   mes_valid       ;Set "valid" message address
        call    ssout           ;Send out message
        movlw   0x0d
        call    sout            ;Send out CR
        movlw   0x0a
        call    sout            ;Send out LF
        return
;
;
;********************************************************************
; Timer ON command
;    Input parameter   None
;    Output parameter  freg0 = 0  : Normal parameter
;                      freg0 < 0  : Parameter error
;    Used register     freg0
;********************************************************************
cmd_timer_on:
        incf    FSR,f                ;Receive data pointer + 1
        cp      INDF,'1'             ;Timer1 ?
        jpnz    cmd_timer_on1        ;No. Next
        bsf     timer1_out           ;Set Timer1 ON
        goto    cmd_timer_on5        ;Jump to message out
cmd_timer_on1:
        cp      INDF,'2'             ;Timer2 ?
        jpnz    cmd_timer_on2        ;No. Next
        bsf     timer2_out           ;Set Timer2 ON
        goto    cmd_timer_on5        ;Jump to message out
cmd_timer_on2:
        cp      INDF,'3'             ;Timer3 ?
        jpnz    cmd_timer_on3        ;No. Next
        bsf     timer3_out           ;Set Timer3 ON
        goto    cmd_timer_on5        ;Jump to message out
cmd_timer_on3:
        cp      INDF,'4'             ;Timer4 ?
        jpnz    cmd_timer_on4        ;No. Next
        bsf     timer4_out           ;Set Timer4 ON
        goto    cmd_timer_on5        ;Jump to message out
cmd_timer_on4:
        bcf     INDF,5               ;Change lowercase to uppercase
        cp      INDF,'A'             ;All timer ?
        jpnz    cmd_timer_on_err     ;No. Jump to error
        bsf     timer1_out           ;Set Timer1 ON
        bsf     timer2_out           ;Set Timer2 ON
        bsf     timer3_out           ;Set Timer3 ON
        bsf     timer4_out           ;Set Timer4 ON
cmd_timer_on5:
        movlw   mes_timer            ;Set "Timer" message address
        call    ssout                ;Send out message
        cp      INDF,'A'             ;All timer ?
        jpnz    cmd_timer_on6        ;No. Jump to cmd_timer_on6
        movlw   ' '                  ;Yes.
        call    sout                 ;Send out Space
        movlw   'A'
        call    sout                 ;Send out 'A'
        movlw   'L'
        call    sout                 ;Send out 'L'
        movlw   'L'
        call    sout                 ;Send out 'L'
        goto    cmd_timer_on7        ;Jump to message out
cmd_timer_on6:
        movf    INDF,w               ;Set Timer number
        call    sout                 ;Send out Timer number
cmd_timer_on7:
        movlw   ' '
        call    sout                 ;Send out Space
        movlw   'O'
        call    sout                 ;Send out 'O'
        movlw   'N'
        call    sout                 ;Send out 'N'
        movlw   0x0d
        call    sout                 ;Send out CR
        movlw   0x0a
        call    sout                 ;Send out LF
        clrf    freg0                ;Set Normal status
        goto    cmd_timer_on_exit
cmd_timer_on_err:
        clrf    freg0
        decf    freg0,f              ;Set parameter error status
cmd_timer_on_exit:
        return
;
;
;********************************************************************
; Timer OFF command
;    Input parameter   None
;    Output parameter  freg0 = 0  : Normal parameter
;                      freg0 < 0  : Parameter error
;    Used register     freg0
;********************************************************************
cmd_timer_off:
        incf    FSR,f                ;Receive data pointer + 1
        cp      INDF,'1'             ;Timer1 ?
        jpnz    cmd_timer_off1       ;No. Next
        bcf     timer1_out           ;Set Timer1 OFF
        goto    cmd_timer_off5       ;Jump to message out
cmd_timer_off1:
        cp      INDF,'2'             ;Timer2 ?
        jpnz    cmd_timer_off2       ;No. Next
        bcf     timer2_out           ;Set Timer2 OFF
        goto    cmd_timer_off5       ;Jump to message out
cmd_timer_off2:
        cp      INDF,'3'             ;Timer3 ?
        jpnz    cmd_timer_off3       ;No. Next
        bcf     timer3_out           ;Set Timer3 OFF
        goto    cmd_timer_off5       ;Jump to message out
cmd_timer_off3:
        cp      INDF,'4'             ;Timer4 ?
        jpnz    cmd_timer_off4       ;No. Next
        bcf     timer4_out           ;Set Timer4 OFF
        goto    cmd_timer_off5       ;Jump to message out
cmd_timer_off4:
        bcf     INDF,5               ;Change lowercase to uppercase
        cp      INDF,'A'             ;All timer ?
        jpnz    cmd_timer_off_err    ;No. Jump to error
        bcf     timer1_out           ;Set Timer1 OFF
        bcf     timer2_out           ;Set Timer2 OFF
        bcf     timer3_out           ;Set Timer3 OFF
        bcf     timer4_out           ;Set Timer4 OFF
cmd_timer_off5:
        movlw   mes_timer            ;Set "Timer" message address
        call    ssout                ;Send out message
        cp      INDF,'A'             ;All timer ?
        jpnz    cmd_timer_off6       ;No. Jump to cmd_timer_off6
        movlw   ' '                  ;Yes.
        call    sout                 ;Send out Space
        movlw   'A'
        call    sout                 ;Send out 'A'
        movlw   'L'
        call    sout                 ;Send out 'L'
        movlw   'L'
        call    sout                 ;Send out 'L'
        goto    cmd_timer_off7        ;Jump to message out
cmd_timer_off6:
        movf    INDF,w               ;Set Timer number
        call    sout                 ;Send out Timer number
cmd_timer_off7:
        movlw   ' '
        call    sout                 ;Send out Space
        movlw   'O'
        call    sout                 ;Send out 'O'
        movlw   'F'
        call    sout                 ;Send out 'F'
        movlw   'F'
        call    sout                 ;Send out 'F'
        movlw   0x0d
        call    sout                 ;Send out CR
        movlw   0x0a
        call    sout                 ;Send out LF
        clrf    freg0                ;Set Normal status
        goto    cmd_timer_off_exit
cmd_timer_off_err:
        clrf    freg0
        decf    freg0,f              ;Set parameter error status
cmd_timer_off_exit:
        return
;
;
;********************************************************************
; Send out error message
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
sout_err:
        movlw   '?'
        call    sout            ;Send out '?'
        movlw   0x0d
        call    sout            ;Send out CR
        movlw   0x0a
        call    sout            ;Send out LF
        return
;
;
;********************************************************************
; Send out one character to serial port
;    Input parameter   W = Send out data address
;    Output parameter  None
;    Used register     None
;********************************************************************
sout:
        btfss   PIR1,TXIF       ;Buffer full ?
        goto    sout            ;Yes. Wait
        movwf   TXREG           ;No. Set data to TXREG register
        return
;
;
;********************************************************************
; Send out characters to serial port
;    Input parameter   W = Send out data (End is 0)
;    Output parameter  None
;    Used register     freg0,freg1
;********************************************************************
ssout:
        movwf   freg1           ;Save send out data address
ssout1:
        lcall   get_tbl_data    ;Read data from data table
        setpch                  ;PCLATH setting macro
        iorlw   0               ;Check W register
        skpnz                   ;W = 0 ? (End ?)
        goto    ssout2          ;Yes. Jump to return
        call    sout            ;No. Send out one character
        incf    freg1,f         ;Table address + 1
        goto    ssout1          ;Jump to ssout1
ssout2:
        return
;
;
;********************************************************************
; Send out second
;    Input parameter   tm_1s,tm_10s
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_second:
        movf    tm_10s,w        ;Read 10th of second
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of second
        movf    tm_1s,w         ;Read 1st of second
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of second
        return
;
;
;********************************************************************
; Send out minute
;    Input parameter   tm_1m,tm_10m
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_minute:
        movf    tm_10m,w        ;Read 10th of minute
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of minute
        movf    tm_1m,w         ;Read 1st of minute
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of minute
        return
;
;
;********************************************************************
; Send out hour
;    Input parameter   tm_1h,tm_10h
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_hour:
        movf    tm_10h,w        ;Read 10th of hour
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of hour
        movf    tm_1h,w         ;Read 1st of hour
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of hour
        return
;
;
;********************************************************************
; Send out day
;    Input parameter   tm_1d,tm_10d
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_day:
        movf    tm_10d,w        ;Read 10th of day
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of day
        movf    tm_1d,w         ;Read 1st of day
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of day
        return
;
;
;********************************************************************
; Send out month
;    Input parameter   tm_1mth,tm_10mth
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_month:
        movf    tm_10mth,w      ;Read 10th of month
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of month
        movf    tm_1mth,w       ;Read 1st of month
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of month
        return
;
;
;********************************************************************
; Send out year
;    Input parameter   tm_1y,tm_10y
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_year:
        movlw   "2"
        call    sout            ;Send out '2'
        movlw   "0"
        call    sout            ;Send out '0'
        movf    tm_10y,w        ;Read 10th of year
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 10th of year
        movf    tm_1y,w         ;Read 1st of year
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st of year
        return
;
;
;********************************************************************
; Send out week
;    Input parameter   tm_wd
;    Output parameter  None
;    Used register     freg0
;********************************************************************
sout_wday:
        clrc                    ;Clear Carry bit
        rlf     tm_wd,w         ;W = tm_wd * 2
        movwf   freg0           ;Save W
        rlf     freg0,w         ;W = tm_wd * 4
        addlw   mes_wday        ;(tm_wd * 4) + week message address
        call    ssout           ;Send out week message
        return
;
;
;********************************************************************
; Change two characters to binary
;    Input parameter   INDF(2) = Characters
;    Output parameter  W = Binary value
;                      freg0 = 0 : Normal
;                      freg0 = 1 : Error
;    Used register     acch,accm,accl,bcch,bccm,bccl
;********************************************************************
cv_str2num:
        clrf    bcch            ;Clear BCCH
        clrf    bccl            ;Clear BCCL
        clrf    freg0           ;Clear freg0
        incf    freg0,f         ;Set parameter error code
        cp      INDF,'9'+1      ;10th > 10 ?
        jpnb    cv_str2num_err  ;Yes. Jump to return
        movlw   '0'
        subwf   INDF,w          ;10th < 0 ?
        jpb     cv_str2num_err  ;Yes. Jump to return
        movwf   accl            ;Set 10th value to ACCH
        movlw   10
        lcall   mulib           ;BCCH,BCCL = BCCH,BCCL + (ACCL * W)
        setpch                  ;PCLATH setting macro
        incf    FSR,f           ;Character pointer + 1
        cp      INDF,'9'+1      ;1st > 10 ?
        jpnb    cv_str2num_err  ;Yes. Jump to return
        movlw   '0'
        subwf   INDF,w          ;1st < 0 ?
        jpb     cv_str2num_err  ;Yes. Jump to return
        addwf   bccl,w          ;BCCL + 1st value
        clrf    freg0           ;Set Normal parameter code
cv_str2num_err:
        return
;
;
;********************************************************************
; Send out two characters
;    Input parameter   W = Send out data
;    Output parameter  None
;    Used register     acch,accl
;********************************************************************
sout_num2d:
        movwf   accl            ;Save send out data
        movlw   10
        lcall   divb            ;Send out data / 10
        setpch                  ;PCLATH setting macro
        movf    acch,w          ;Read Quotient
        addlw	'0'             ;Change to character code
        call    sout            ;Send out 10th
        movf    accl,w          ;Read Rest
        addlw   '0'             ;Change to character code
        call    sout            ;Send out 1st
        return
;
;
;********************************************************************
; Check EEPROM data
;    Input parameter   None
;    Output parameter  None
;    Used register     eep_add,eep_data,freg1
;********************************************************************
chk_eep_dat:
        rbank1                  ;Set register bank 1
        clrf    eep_add         ;Clear EEP_ADD
chk_eep_dat1:
        call    rd_eeprom       ;Read EEPROM data
        movf    eep_add,w       ;Read EEPROM address
        addlw   mes_open1       ;EEPROM address + Open message address
        rbank0                  ;Set register bank 0
        movwf   freg1           ;Save address
        lcall   get_tbl_data    ;Read message
        setpch                  ;PCLATH setting macro
        rbank1                  ;Set register bank 1
        cpw     eep_data        ;Check EEPROM data
        jpnz    chk_eep_dat2    ;If no data on EEPROM, jump to chk_eep_dat2
        incf    eep_add,f       ;EEPROM address + 1
        cp      eep_add,15      ;All checked ?
        jpnz    chk_eep_dat1    ;No. Jump to chk_eep_dat1
        goto    chk_eep_dat3    ;Yes. Jump to chk_eep_dat3
chk_eep_dat2:
        call    eeprom_init     ;Initialize EEPROM
chk_eep_dat3:
        rbank0                  ;Set register bank 0
        return
;
;
;********************************************************************
; Initialize EEPROM
;    Input parameter   None
;    Output parameter  None
;    Used register     freg5
;********************************************************************
eeprom_init:
        clrf    eep_add             ;Clear EEP_ADD
eeprom_init1:
        movf    eep_add,w           ;Read EEPROM address
        addlw   mes_open1           ;EEPROM address + Open message address
        rbank0                      ;Set register bank 0
        movwf   freg1               ;Save address
        lcall   get_tbl_data        ;Read message
        setpch                      ;PCLATH setting macro
        rbank1                      ;Set register bank 1
        movwf   eep_data            ;Set EEPROM data
        call    wr_eeprom           ;Write data to EEPROM
        incf    eep_add,f           ;EEPROM address + 1
        cp      eep_add,15          ;All done ?
        jpnz    eeprom_init1        ;No. Jump to eeprom_init1
;
        movlw   0x10
        movwf   eep_add             ;Set EEPROM address 16(10h)
        movlw   29                  ;Set loop count(12/24,timer * 4)
        movwf   freg10              ;Save count
        clrf    eep_data            ;Clear EEPROM data
eeprom_init2:
        call    wr_eeprom           ;Write data to EEPROM
        incf    eep_add,f           ;EEPROM address + 1
        djnz    freg10,eeprom_init2 ;If not finish, jump to eeprom_init2
        return
;
;
;********************************************************************
; Read EEPROM data
;    Input parameter   eep_add  = Address
;    Output parameter  eep_data = Read data
;    Used register     None
;********************************************************************
rd_eeprom:
        movf    eep_add,w       ;Read EEPROM address
        rbank2                  ;Set register bank 2
        movwf   EEADR           ;Set EEPROM address to EEADR register
        rbank3                  ;Set register bank 3
        bcf     EECON1,EEPGD    ;Set data memory access
        bsf     EECON1,RD       ;Set memory read strat
        rbank2                  ;Set register bank 2
        movf    EEDATA,w        ;Read EEPROM data
        rbank1                  ;Set register bank 1
        movwf   eep_data        ;Set EEPROM data
        return
;
;
;********************************************************************
; Write data to EEPROM
;    Input parameter   eep_add  = Address
;                      eep_data = Data
;    Output parameter  None
;    Used register     None
;********************************************************************
wr_eeprom:
        movf    eep_add,w       ;Read EEPROM address
        rbank2                  ;Set register bank 2
        movwf   EEADR           ;Set EEPROM address to EEADR register
        rbank1                  ;Set register bank 1
        movf    eep_data,w      ;Read data
        rbank2                  ;Set register bank 2
        movwf   EEDATA          ;Set data to EEDATA register
        rbank3                  ;Set register bank 3
        bcf     EECON1,EEPGD    ;Set data memory access
        bsf     EECON1,WREN     ;Set write enable
        di                      ;Disable interrupt
        movlw   0x55
        movwf   EECON2          ;Set write protection key(55h)
        movlw   0xaa
        movwf   EECON2          ;Set write protection key(AAh)
        bsf     EECON1,WR       ;Set WR bit to begin write
        ei                      ;Enable interrupt
wr_eeprom1:
        btfsc   EECON1,WR       ;Finish writing ?
        goto    wr_eeprom1      ;No. Wait
        bcf     EECON1,WREN     ;Yes. Set write disable
        rbank1                  ;Set register bank 1
        return
;
;
;********************************************************************
; Check time of Timer
;    Input parameter   None
;    Output parameter  None
;    Used register     tmbf_mode(7),eep_add,eep_data,freg0,tmp_FSR2
;********************************************************************
chk_timer:
        movf    FSR,w           ;Read FSR
        movwf   tmp_FSR2        ;Save FSR
;
        movlw   ead_tm1_mode    ;Set Timer1 mode EEPROM address
        call    get_tmbf_data   ;Read timer data
        movf    tmbf_mode,f     ;Timer1 mode = OFF ?
        jpz     chk_timer2      ;Yes. Jump next
        movlw   tmbf_str_hour   ;Set timer start hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer1 ON ?
        jpz     chk_timer1      ;No. Jump to chk_timer1
        bsf     timer1_out      ;Yes. Set Timer1 ON bit
chk_timer1:
        movlw   tmbf_stp_hour   ;Set timer stop hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer1 OFF ?
        jpz     chk_timer2      ;No. Jump to chk_timer2
        bcf     timer1_out      ;Yes. Set Timer1 OFF bit
        btfss   tmbf_mode,1     ;Timer mode = SINGLE ?
        goto    chk_timer2      ;If CONTINUE, jump to chk_timer2
        rbank1                  ;Set register bank 1
        movlw   ead_tm1_mode
        movwf   eep_add         ;Set Timer1 mode EEPROM address
        clrf    eep_data        ;Clear data
        call    wr_eeprom       ;Write Timer1 OFF
        rbank0                  ;Set register bank 0
chk_timer2:
        movlw   ead_tm2_mode    ;Set Timer2 mode EEPROM address
        call    get_tmbf_data   ;Read timer data
        movf    tmbf_mode,f     ;Timer2 mode = OFF ?
        jpz     chk_timer4      ;Yes. Jump next
        movlw   tmbf_str_hour   ;Set timer start hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer2 ON ?
        jpz     chk_timer3      ;No. Jump to chk_timer3
        bsf     timer2_out      ;Yes. Set Timer2 ON bit
chk_timer3:
        movlw   tmbf_stp_hour   ;Set timer stop hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer2 OFF ?
        jpz     chk_timer4      ;No. Jump to chk_timer4
        bcf     timer2_out      ;Yes. Set Timer2 OFF bit
        btfss   tmbf_mode,1     ;Timer mode = SINGLE ?
        goto    chk_timer4      ;If CONTINUE, jump to chk_timer4
        rbank1                  ;Set register bank 1
        movlw   ead_tm2_mode
        movwf   eep_add         ;Set Timer2 mode EEPROM address
        clrf    eep_data        ;Clear data
        call    wr_eeprom       ;Write Timer2 OFF
        rbank0                  ;Set register bank 0
chk_timer4:
        movlw   ead_tm3_mode    ;Set Timer3 mode EEPROM address
        call    get_tmbf_data   ;Read timer data
        movf    tmbf_mode,f     ;Timer3 mode = OFF ?
        jpz     chk_timer6      ;Yes. Jump next
        movlw   tmbf_str_hour   ;Set timer start hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer3 ON ?
        jpz     chk_timer5      ;No. Jump to chk_timer5
        bsf     timer3_out      ;Yes. Set Timer3 ON bit
chk_timer5:
        movlw   tmbf_stp_hour   ;Set timer stop hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer3 OFF ?
        jpz     chk_timer6      ;No. Jump to chk_timer6
        bcf     timer3_out      ;Yes. Set Timer3 OFF bit
        btfss   tmbf_mode,1     ;Timer mode = SINGLE ?
        goto    chk_timer6      ;If CONTINUE, jump to chk_timer6
        rbank1                  ;Set register bank 1
        movlw   ead_tm3_mode
        movwf   eep_add         ;Set Timer3 mode EEPROM address
        clrf    eep_data        ;Clear data
        call    wr_eeprom       ;Write Timer3 OFF
        rbank0                  ;Set register bank 0
chk_timer6:
        movlw   ead_tm4_mode    ;Set Timer4 mode EEPROM address
        call    get_tmbf_data   ;Read timer data
        movf    tmbf_mode,f     ;Timer4 mode = OFF ?
        jpz     chk_timer8      ;Yes. Jump to return
        movlw   tmbf_str_hour   ;Set timer start hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer4 ON ?
        jpz     chk_timer7      ;No. Jump to chk_timer7
        bsf     timer4_out      ;Yes. Set Timer4 ON bit
chk_timer7:
        movlw   tmbf_stp_hour   ;Set timer stop hour address
        call    comp_timer      ;Check time
        movf    freg0,f         ;Present time = Timer4 OFF ?
        jpz     chk_timer8      ;No. Jump to return
        bcf     timer4_out      ;Yes. Set Timer4 OFF bit
        btfss   tmbf_mode,1     ;Timer mode = SINGLE ?
        goto    chk_timer8      ;If CONTINUE, jump to return
        rbank1                  ;Set register bank 1
        movlw   ead_tm4_mode
        movwf   eep_add         ;Set Timer4 mode EEPROM address
        clrf    eep_data        ;Clear data
        call    wr_eeprom       ;Write Timer4 OFF
        rbank0                  ;Set register bank 0
chk_timer8:
        movf    tmp_FSR2,w
        movwf   FSR             ;Recover FSR
        return
;
;
;********************************************************************
; Timer OFF
;    Input parameter   None
;    Output parameter  None
;    Used register     None
;********************************************************************
timer_off:
        bcf     timer1_out      ;Set Timer1 OFF
        bcf     timer2_out      ;Set Timer2 OFF
        bcf     timer3_out      ;Set Timer3 OFF
        bcf     timer4_out      ;Set Timer4 OFF
        movf    porta_buf,w
        movwf   PORTA           ;Write PORTA
        return
;
;
;********************************************************************
; Compare Timer time and Present time
;    Input parameter   W = Timer address
;    Output parameter  freg0 = 0 : Mismatch
;                      freg0 = 1 : Match
;    Used register     None
;********************************************************************
comp_timer:
        movwf   FSR             ;Set Timer address to FSR
        clrf    freg0           ;Clear freg0
        movf    INDF,w          ;Read hour
        movwf   accl            ;Set hour to ACCL
        movlw   10
        lcall   divb            ;Hour / 10
        setpch                  ;PCLATH setting macro
        movf    acch,w          ;Read 10th of hour of Timer
        cpw     tm_10h          ;Compare Present 10th of hour
        jpnz    comp_timer1     ;If mismatch, jump to return
        movf    accl,w          ;Read 1st of hour of Timer
        cpw     tm_1h           ;Compare Present 1st of hour
        jpnz    comp_timer1     ;If mismatch, jump to return
        incf    FSR,f           ;Timer address + 1
        movf    INDF,w          ;Read minute
        movwf   accl            ;Set minute to ACCL
        movlw   10
        lcall   divb            ;Minute / 10
        setpch                  ;PCLATH setting macro
        movf    acch,w          ;Read 10th of minute of Timer
        cpw     tm_10m          ;Compare Present 10th of minute
        jpnz    comp_timer1     ;If mismatch, jump to return
        movf    accl,w          ;Read 1st of minute of Timer
        cpw     tm_1m           ;Compare Present 1st of minute
        jpnz    comp_timer1     ;If mismatch, jump to return
        incf    FSR,f           ;Timer address + 1
        movf    INDF,w          ;Read second
        movwf   accl            ;Set second to ACCL
        movlw   10
        lcall   divb            ;Second / 10
        setpch                  ;PCLATH setting macro
        movf    acch,w          ;Read 10th of second of Timer
        cpw     tm_10s          ;Compare Present 10th of second
        jpnz    comp_timer1     ;If mismatch, jump to return
        movf    accl,w          ;Read 1st of second of Timer
        cpw     tm_1s           ;Compare Present 1st of second
        jpnz    comp_timer1     ;If mismatch, jump to return
        incf    freg0,f         ;Set Match code
comp_timer1:
        return
;
;
;********************************************************************
; Set timer data to EEPROM
;    Input parameter   W = Head address of timer data(EEPROM)
;                      tmbf_mode(7) = Timer data
;                         Mode/Strat hour/Strat minute/Start second/
;                         Stop hour/Stop minute/Stop second
;    Output parameter  None
;    Used register     freg10
;********************************************************************
set_tmbf_data:
        rbank1                         ;Set register bank 1
        movwf   eep_add                ;Set EEPROM head address of timer
        movlw   7
        movwf   freg10                 ;Set loop counter
        movlw   tmbf_mode
        movwf   FSR                    ;Set data address
set_tmbf_data1:
        movf    INDF,w                 ;Read data
        movwf   eep_data               ;Set data
        call    wr_eeprom              ;Write data to EEPROM
        incf    eep_add,f              ;EEPROM address + 1
        incf    FSR,f                  ;Data address + 1
        djnz    freg10,set_tmbf_data1  ;If not finish, jump to set_tmbf_data1
        rbank0                         ;Set register bank 0
        return
;
;
;********************************************************************
; Read timer data from EEPROM
;    Input parameter   W = Head address of timer data(EEPROM)
;    Output parameter  tmbf_mode(7) = Timer data
;                         Mode/Strat hour/Strat minute/Start second/
;                         Stop hour/Stop minute/Stop second
;    Used register     freg10
;********************************************************************
get_tmbf_data:
        rbank1                         ;Set register bank 1
        movwf   eep_add                ;Set EEPROM head address of timer
        movlw   7
        movwf   freg10                 ;Set loop counter
        movlw   tmbf_mode
        movwf   FSR                    ;Set data address
get_tmbf_data1:
        call    rd_eeprom              ;Read timer data from EEPROM
        movf    eep_data,f             ;??
        movwf   INDF                   ;Set timer data to tmbf_mode
        incf    eep_add,f              ;EEPROM address + 1
        incf    FSR,f                  ;Data address + 1
        djnz    freg10,get_tmbf_data1  ;If not finish, jump to get_tmbf_data1
        rbank0                         ;Set register bank 0
        return
;
;
;********************************************************************
; Read designated timer data from EEPROM
;    Input parameter   cur_timer  = Timer number
;                      tm_str_stp = 0 : Start time
;                      tm_str_stp = 1 : Stop time
;    Output parameter  tmbf_mode(4) = Timer data
;                         Mode/Hour/Minute/Second
;    Used register     freg10
;********************************************************************
get_tm_data:
        movlw   tm_mode
        movwf   FSR                    ;Set tm_mode address
        movf    cur_timer,w            ;Read Timer number
        rbank1                         ;Set register bank 1
        movwf   eep_add                ;Save Timer number
        movwf   freg10                 ;Save Timer number
        clrc                           ;Clear Carry bit
        rlf     freg10,f               ;Timer number * 2
        rlf     freg10,w               ;Timer number * 4
        addwf   freg10,w               ;Timer number * 6
        addwf   eep_add,f              ;Timer number * 7
        movlw   ead_tm1_mode           ;Set Timer1 mode address on EEPROM
        addwf   eep_add,f              ;Set EEPROM address
        call    rd_eeprom              ;Read EEPROM data
        incf    eep_add,f              ;Set timer start hour address
        movf    eep_data,w             ;Read mode data
        movwf   INDF                   ;Set mode data
        rbank0                         ;Set register bank 0
        btfss   tm_str_stp             ;Start time ?
        goto    get_tm_data1           ;Yes. Jump to get_tm_data1
        rbank1                         ;No. Set register bank 1
        movlw   3
        addwf   eep_add,f              ;Set timer stop hour address
get_tm_data1:
        rbank1                         ;Set register bank 1
        incf    FSR,f                  ;Set hour data address
        movlw   3
        movwf   freg10                 ;Set loop counter
get_tm_data2:
        call    rd_eeprom              ;Read EEPROM data
        movf    eep_data,w             ;Read data
        movwf   INDF                   ;Set data
        incf    eep_add,f              ;EEPROM address + 1
        incf    FSR,f                  ;Data address + 1
        djnz    freg10,get_tm_data2    ;If not finish, jump to get_tm_data2
        rbank0                         ;Set register bank 0
        return
;
;
;********************************************************************
; Write designated timer data to EEPROM
;    Input parameter   cur_timer  = Timer number
;                      tm_str_stp = 0 : Start time
;                      tm_str_stp = 1 : Stop time
;                      tmbf_mode(4) = Timer data
;                         Mode/Hour/Minute/Second
;    Output parameter  None
;    Used register     freg10
;********************************************************************
set_tm_data:
        movlw   tm_mode
        movwf   FSR                    ;Set tm_mode address
        movf    cur_timer,w            ;Read Timer number
        rbank1                         ;Set register bank 1
        movwf   eep_add                ;Save Timer number
        movwf   freg10                 ;Save Timer number
        clrc                           ;Clear Carry bit
        rlf     freg10,f               ;Timer number * 2
        rlf     freg10,w               ;Timer number * 4
        addwf   freg10,w               ;Timer number * 6
        addwf   eep_add,f              ;Timer number * 7
        movlw   ead_tm1_mode           ;Set Timer1 mode address on EEPROM
        addwf   eep_add,f              ;Set EEPROM address
        movf    INDF,w                 ;Read mode data
        movwf   eep_data               ;Set mode data
        call    wr_eeprom              ;Write EEPROM
        incf    eep_add,f              ;Set hour data address
        rbank0                         ;Set register bank 1
        btfss   tm_str_stp             ;Start time ?
        goto    set_tm_data1           ;Yes. Jump to set_tm_data1
        rbank1                         ;No. Set register bank 1
        movlw   3
        addwf   eep_add,f              ;Set timer stop hour address
set_tm_data1:
        rbank1                         ;Set register bank 1
        incf    FSR,f                  ;Set hour data address
        movlw   3
        movwf   freg10                 ;Set loop counter
set_tm_data2:
        movf    INDF,w
        movwf   eep_data               ;Set data
        call    wr_eeprom              ;Write data
        incf    eep_add,f              ;EEPROM address + 1
        incf    FSR,f                  ;Data address + 1
        djnz    freg10,set_tm_data2    ;If not finish, jump to set_tm_data2
        rbank0                         ;Set register bank 0
        return
;
;
;********************************************************************
; Set upper limit of timer
;    Input parameter   tm_index = Timer item index
;    Output parameter  tm_limit = Upper limit data
;    Used register     freg1
;********************************************************************
set_tm_lim:
        movf    tm_index,w      ;Read itme index
        addlw   tm_lim_tbl
        movwf   freg1           ;Set table address
        lcall   get_tbl_data    ;Read data
        setpch                  ;PCLATH setting macro
        movwf   tm_limit        ;Set upper limit data
        return
;
;
;####################################################################
;
;                          Initial routine
;
;####################################################################
init:
        setpch                  ;Set PCLATH register
        rbank0                  ;Set bank 0
        clrf    PORTA           ;Clear PORTA
        clrf    PORT_B          ;Clear PORTB
        clrf    PORTC           ;Clear PORTC
        movlw   0x90            ;Serial,8bit mode,Continuous receive
        movwf   RCSTA           ;Set RCSTA register
        movlw   0x00            ;A/D disable
        movwf   ADCON0          ;Set ADCON0 register
;
        rbank1                  ;Set bank 1
        movlw   0x06            ;PORTA all Digital mode
        movwf   ADCON1          ;Set ADCON1 register
        movlw   0x10            ;RA4(TCO) input mode
        movwf   TRISA           ;Set TRISA register
        movlw   0x0c            ;RB2(MENU),RB3(F1) input mode
        movwf   TRIS_B          ;Set TRISB register
        movlw   0xc0            ;RC6(TX),RC7(RX) input mode
        movwf   TRISC           ;Set TRISC register
        movlw   0x00            ;RBPU, TMR0 sysclk, 1/2
        movwf   OPTION_REG      ;Set OPTION_REG register
        movlw   82              ;9600 bps
        movwf   SPBRG           ;Set SPBRG register
        movlw   0x26            ;8bit, TXEN, Async, High speed
        movwf   TXSTA           ;Set TXSTA register
;
        rbank0                  ;Set bank 0
        clrf    tco_valid       ;Clear TCO valid flag
        clrf    mark_pd_cnt     ;Clear Marker period
        clrf    mark_cnt        ;Clear Marker counter
        clrf    zero_cnt        ;Clear Zero counter
        clrf    dec_sts1        ;Clear Decode status1
        clrf    dec_sts2        ;Clear Decode status2
        clrf    tm_chk_cnt_m    ;Clear Time check counter (Munite)
        clrf    tm_chk_cnt_h    ;Clear Time check counter (Hour)
        clrf    tm_chk_cnt_nd   ;Clear Time check counter (Day)
        clrf    tm_chk_cnt_wd   ;Clear Time check counter (Week)
        clrf    tm_chk_cnt_y    ;Clear Time check counter (Year)
        clrf    dec_min_cnt     ;Clear Decoded time (Every minute)
        lcall   init_time       ;Initialization of the time
        setpch                  ;Set PCLATH register
        clrf    porta_buf       ;Clear PortA buffer
        clrf    portc_buf       ;Clear PortC buffer
        clrf    tled_cnt        ;Clear LED turn on counter
        clrf    err_cnth        ;Clear error counter
        clrf    err_cntl        ;Clear error counter
        clrf    rx_sts          ;Clear reciving flag
        clrf    rx_cc           ;Clear reciving character counter
        movlw   rx_dat          ;Set reciving buffer head address
        movwf   rx_ptr          ;Set reciving buffer pointer
        clrf    key_sts         ;Clear key status
        clrf    key_dat         ;Clear key number
        clrf    k_tmout_cnt     ;Clear key time-out counter
        clrf    disp_flag       ;Clear display control flag
        movlw   6
        movwf   unadj_cnt       ;Set unadjust counter
        rbank1                  ;Set bank 1
        clrf    int_cnt         ;Clear 100uS interval counter
        clrf    int_cnt10m      ;Clear 10mS interval counter
        clrf    int_cnt100m     ;Clear 100mS interval counter
        clrf    tco_cnt         ;Clear TCO counter
        clrf    tco_hicnt       ;Clear TCO high counter
        clrf    tco_period      ;Clear TCO period counter
        clrf    tco_sts         ;Clear TCO status
        clrf    sync_cnt        ;Clear Sync valid counter
        clrf    ikey_dat        ;Clear Key number
        clrf    ikey_dat0       ;Clear Key scan position
        clrf    ikey_dat1       ;Clear Key input data
        clrf    key_phs         ;Clear Key phase
        clrf    rep_cnt         ;Clear key repeate counter
        rbank0                  ;Set bank 0
        call    chk_eep_dat     ;Check EEPROM data
        rbank1                  ;Set bank 1
        movlw   ead_disp_mode   ;Set display mode addres on EEPROM
        movwf   eep_add         ;Set EEPROM address
        call    rd_eeprom       ;Read EEPROM data
        movf    eep_data,w      ;Read display mode data
        rbank0                  ;Set bank 0
        movwf   disp_mode       ;Set display mode data
;
        movlw   b'00000110'     ;1/1 post, TMR2 on, 1/16 pre
        movwf   T2CON           ;Set T2CON
        clrf    TMR2            ;Clear TMR2
        rbank1                  ;Set bank 1
        movlw   19              ;Set period(100uS)
        movwf   PR2             ;TMR2 period value
        movlw   b'00100010'     ;RCIE TMR2IE
        movwf   PIE1            ;Set PIE1
        rbank0                  ;Set bank 0
        movlw   b'11000000'     ;PE int enable
        movwf   INTCON          ;Set INTCON
        lcall   lcd_init        ;Initialization of the LCD module
        setpch                  ;Set PCLATH register
;
        goto    main
;
;
;####################################################################
;
;                           Main routine
;
;####################################################################

main:
        movlw   mes_open1
        lcall   messout                 ;Display "JJY Radio Clock" message
        movlw   2                       ;??
        clrf    xpos                    ;Clear X positon
        incf    ypos,f                  ;Y position + 1
        lcall   locate                  ;Change cursor location
        movlw   mes_open2
        lcall   messout                 ;Display "Ver.2.02" message
        movlw   200                     ;Set wait value
        lcall   delay10m                ;2 seconds delay
        lcall   cls                     ;Clear display
        setpch                          ;Set PCLATH register
;
main1:
        call    clock_main              ;Clock main process
        btfss   key_sts,0               ;Key pressed ?
        goto    main1                   ;No. Jump to main1
        clrf    key_sts                 ;Yes. Clear key status
        cp      key_dat,menu_key        ;Menu key ?
        jpz     key_job                 ;Yes. Jump to Menu key process
        cp      key_dat,tm_off_key      ;Timer OFF key ?
        jpnz    main1                   ;No. Jump to main1
        call    timer_off               ;Timer OFF
        goto    main1                   ;Jump to main1
;
key_job:                                ;Menu key process
        movlw   k_tmout_v               ;Read key time-out (10S)
        di                              ;Disable interrupt
        movwf   k_tmout_cnt             ;Set key time-out counter
        ei                              ;Enable interrupt
        bsf     disp_inh                ;Set time display inhibit (1:Stop)
        lcall   cls                     ;Clear display
        setpch                          ;Set PCLATH register
        clrf    xpos                    ;Clear X positon
        movlw   1
        movwf   ypos                    ;Set Y to 2nd line
        lcall   locate                  ;Change cursor location
        setpch
        movlw   mes_TM
        lcall   messout                 ;Display "TM" message
        setpch
        movlw   '1'
        lcall   print                   ;Display "1"
        setpch
        movlw   ' '
        lcall   print                   ;Display Space
        setpch
        movlw   mes_TM
        lcall   messout                 ;Display "TM" message
        setpch
        movlw   '2'
        lcall   print                   ;Display "2"
        setpch
        movlw   ' '
        lcall   print                   ;Display Space
        setpch
        movlw   mes_TM
        lcall   messout                 ;Display "TM" message
        setpch
        movlw   '3'
        lcall   print                   ;Display "3"
        setpch
        movlw   ' '
        lcall   print                   ;Display Space
        setpch
        movlw   mes_TM
        lcall   messout                 ;Display "TM" message
        setpch
        movlw   '4'
        lcall   print                   ;Display "4"
        setpch
        movlw   ' '
        lcall   print                   ;Display Space
        setpch
key_job1:
        call    clock_main              ;Clock main process
        movf    k_tmout_cnt,f           ;Key time-out ?
        jpz     key_job4                ;Yes. Jump to key end
        btfss   key_sts,0               ;Key pressed ?
        goto    key_job1                ;No. wait
        clrf    key_sts                 ;Yes. Clear key status
        cp      key_dat,menu_key        ;Menu key pressed ?
        jpz     key_job2                ;Yes. Jump to Menu key process
        cp      key_dat,f1_key          ;F1 key pressed ?
        jpz     menu_timer1             ;Yes. Jump to TM1 process
        cp      key_dat,f2_key          ;F2 key pressed ?
        jpz     menu_timer2             ;Yes. Jump to TM2 process
        cp      key_dat,f3_key          ;F3 key pressed ?
        jpz     menu_timer3             ;Yes. Jump to TM3 process
        cp      key_dat,f4_key          ;F4 key pressed ?
        jpz     menu_timer4             ;Yes. Jump to TM3 process
        cp      key_dat,tm_off_key      ;Timer OFF key pressed ?
        jpnz    key_job1                ;No. wait
        call    timer_off               ;Yes. All Timer OFF
        goto    key_job1                ;Jump to key_job1
;
key_job2:                               ;Menu + Menu
        lcall   cls                     ;Clear display
        setpch
        clrf    xpos                    ;Clear X positon
        movlw   1
        movwf   ypos                    ;Set Y to 2nd line
        lcall   locate                  ;Change cursor location
        setpch
        movlw   mes_12H                 ;Set "12H"
        btfsc   d12_24                  ;24 hour mode ? (0:24H/1:12H)
        movlw   mes_24H                 ;Yes. Set "24H"
        lcall   messout                 ;Display message
        setpch
key_job3:
        call    clock_main              ;Clock main process
        movf    k_tmout_cnt,f           ;Key time-out ?
        jpz     key_job4                ;Yes. Jump to key end
        btfss   key_sts,0               ;Key pressed ?
        goto    key_job3                ;No. wait
        clrf    key_sts                 ;Yes. Clear key status
        cp      key_dat,menu_key        ;Menu key pressed ?
        jpz     key_job4                ;Yes. Jump to key end
        cp      key_dat,f1_key          ;F1 key pressed ?
        jpz     menu_disp_12_24         ;Yes. Jump to change 12H/24H
        cp      key_dat,tm_off_key      ;Timer OFF key pressed ?
        jpnz    key_job3                ;No. wait
        call    timer_off               ;Yes. All Timer OFF
        goto    key_job3                ;Jump to key_job3
;
key_job4:                               ;Key end precess
        lcall   cls                     ;Clear display
        setpch
        bcf     disp_inh                ;Clear time display inhibit (0:Display)
        call    disp_main               ;Time display
        goto    main1                   ;Jump to main
;
menu_disp_12_24:                        ;Change 12H/24H
        btfss   d12_24                  ;24 hour mode ? (0:24H/1:12H)
        goto    menu_disp_12_24_1       ;Yes. Jump to 12H mode setting
        bcf     d12_24                  ;No. Set 24H mode
        goto    menu_disp2              ;Jump to menu_disp2
menu_disp_12_24_1:
        bsf     d12_24                  ;Set 12H mode
        goto    menu_disp2              ;Jump to menu_disp2
;
menu_disp2:
        movf    disp_mode,w             ;Read display mode
        rbank1
        movwf   eep_data                ;Set EEPROM data
        movlw   ead_disp_mode
        movwf   eep_add                 ;Set EEPROM address
        call    wr_eeprom               ;Write display mode to EEPROM
        rbank0
        goto    key_job4                ;Jump to key end
;
menu_timer1:
        movlw   TM1                     ;Set TM1 index(0)
        goto    menu_tm_set
menu_timer2:
        movlw   TM2                     ;Set TM2 index(1)
        goto    menu_tm_set
menu_timer3:
        movlw   TM3                     ;Set TM3 index(2)
        goto    menu_tm_set
menu_timer4:
        movlw   TM4                     ;Set TM4 index(3)
;
menu_tm_set:
        movwf   cur_timer               ;Savw current timer index
        movlw   k_tmout_v               ;Read key time-out (10S)
        di                              ;Disable interrupt
        movwf   k_tmout_cnt             ;Set key time-out counter
        ei                              ;Enable interrupt
        bsf     disp_inh                ;Set time display inhibit (1:Stop)
        lcall   cls                     ;Clear display
        setpch
        movlw   mes_timer
        lcall   messout                 ;Display "Timer" message
        setpch
        movlw   ' '
        lcall   print                   ;Display Space
        setpch
        movf    cur_timer,w             ;Read current timer index
        addlw   '1'
        lcall   print                   ;Display Timer number
        setpch
        clrf    xpos                    ;Clear X positon
        movlw   1
        movwf   ypos                    ;Set Y to 2nd line
        lcall   locate                  ;Change cursor location
        setpch
        movlw   mes_START
        lcall   messout                 ;Display "STR " message
        setpch
        movlw   mes_STOP
        lcall   messout                 ;Display "STP " message
        setpch
menu_tm_set1:
        call    clock_main              ;Clock main process
        movf    k_tmout_cnt,f           ;Key time-out ?
        jpz     key_job4                ;Yes. Jump to key end
        btfss   key_sts,0               ;Key pressed ?
        goto    menu_tm_set1            ;No. wait
        clrf    key_sts                 ;Yes. Clear key status
        cp      key_dat,menu_key        ;Menu key pressed ?
        jpz     key_job4                ;Yes. Jump to key end
        cp      key_dat,f1_key          ;F1 key pressed ?
        jpz     menu_start_time         ;Yes. Jump to timer setting
        cp      key_dat,f2_key          ;F2 key pressed ?
        jpz     menu_stop_time          ;Yes. Jump to timer stopping
        cp      key_dat,tm_off_key      ;Timer OFF key pressed ?
        jpnz    menu_tm_set1            ;No. wait
        call    timer_off               ;Yes. All Timer OFF
        goto    menu_tm_set1            ;Jump to menu_tm_set1
;
menu_tm_set2: ;These steps aren't used. They are the same as "key_job4:"
        lcall   cls                     ;Clear display
        setpch
        bcf     disp_inh                ;Clear time display inhibit (0:Display)
        call    disp_main               ;Time display
        goto    main1                   ;Jump to main
;
menu_start_time:
        bcf     tm_str_stp              ;Set timer start (0:Start/1:Stop)
        goto    menu_tm_time
menu_stop_time:
        bsf     tm_str_stp              ;Set timer stop (0:Start/1:Stop)
;
menu_tm_time:
        call    get_tm_data             ;Read timer data from EEPROM
        bsf     disp_inh                ;Set time display inhibit (1:Stop)
        lcall   cls                     ;Clear display
        setpch
        lcall   disp_timer              ;"TMX XXX XX:XX:XX"
        clrf    xpos                    ;Clear X positon
        movlw   1
        movwf   ypos                    ;Set Y to 2nd line
        lcall   locate                  ;Change cursor location
        setpch
        movlw   mes_menu_TMset
        lcall   messout                 ;" <-   ->  UP  DWN "
        clrf    tm_index                ;Clear Timer setting item index
        lcall   disp_tm_set             ;Timer set value display
        setpch
        movlw   tm_mode
        movwf   FSR                     ;Set timer set data address(RAM)
        call    set_tm_lim              ;Set upper limit of timer
;
menu_tm_time1:
        movlw   k_tmout_v               ;Read key time-out (10S)
        di                              ;Disable interrupt
        movwf   k_tmout_cnt             ;Set key time-out counter
        ei                              ;Enable interrupt
menu_tm_time2:
        call    clock_main              ;Clock main process
        movf    k_tmout_cnt,f           ;Key time-out ?
        jpz     menu_tm_time3           ;Yes. Jump to key end
        btfss   key_sts,0               ;Key pressed ?
        goto    menu_tm_time2           ;No. wait
        clrf    key_sts                 ;Yes. Clear key status
        cp      key_dat,menu_key        ;Menu key pressed ?
        jpz     menu_tm_time3           ;Yes. Jump to key end
        cp      key_dat,f1_key          ;F1 key pressed ?
        jpz     menu_tm_left            ;Yes. Jump to Left key process
        cp      key_dat,f2_key          ;F2 key pressed ?
        jpz     menu_tm_right           ;Yes. Jump to Right key process
        cp      key_dat,f3_key          ;F3 key pressed ?
        jpz     menu_tm_up              ;Yes. Jump to UP key process
        cp      key_dat,f4_key          ;F4 key pressed ?
        jpz     menu_tm_down            ;Yes. Jump to DOWN key process
        cp      key_dat,tm_off_key      ;Timer OFF key pressed ?
        jpnz    key_job4                ;No. Jump to key end
        call    timer_off               ;Yes. All Timer OFF
        goto    menu_tm_time2           ;Jump to menu_tm_time2
menu_tm_time3:
        call    set_tm_data             ;Write time to EEPROM
        lcall   csr_off                 ;Cursor OFF
        setpch
        goto    key_job4                ;Jump to key end
;
menu_tm_left:
        movf    tm_index,f              ;Timer setting item index = 0 ?
        jpnz    menu_tm_left1           ;No. Jump to menu_tm_left1
        movlw   max_tm_index
        movwf   tm_index                ;Set Max item index (3)
        movlw   tm_second
        movwf   FSR                     ;Set second data address(RAM)
        goto    menu_tm_left2
menu_tm_left1:
        decf    FSR,f                   ;Address - 1
        decf    tm_index,f              ;Item index - 1
menu_tm_left2:
        lcall   disp_tm_set             ;Display timer value
        setpch
        call    set_tm_lim              ;Set upper limit
        goto    menu_tm_time1           ;Jump to menu_tm_time1
;
menu_tm_right:
        cp      tm_index,max_tm_index   ;Item index = Max(3) ?
        jpb     menu_tm_right1          ;No.
        clrf    tm_index                ;Set item index = 0
        movlw   tm_mode
        movwf   FSR                     ;Set mode data address(RAM)
        goto    menu_tm_right2
menu_tm_right1:
        incf    FSR,f                   ;Address + 1
        incf    tm_index,f              ;Item index + 1
menu_tm_right2:
        lcall   disp_tm_set             ;Display timer value
        setpch
        call    set_tm_lim              ;Set upper limit of timer
        goto    menu_tm_time1
;
menu_tm_up:
        movf    tm_limit,w              ;Read upper limit
        cpw     INDF                    ;Over ?
        jpnz    menu_tm_up1             ;No.
        clrf    INDF                    ;Value = 0
        goto    menu_tm_up2
menu_tm_up1:
        incf    INDF,f                  ;Value + 1
menu_tm_up2:
        lcall   disp_tm_set             ;Display timer value
        setpch
        goto    menu_tm_time1
;
menu_tm_down:
        movf    INDF,f                  ;Value = 0 ?
        jpnz    menu_tm_down1           ;No.
        movf    tm_limit,w              ;Read upper limit
        movwf   INDF                    ;Set upper value
        goto    menu_tm_down2
menu_tm_down1:
        decf    INDF,f                  ;Value - 1
menu_tm_down2:
        lcall   disp_tm_set             ;Display timer value
        setpch
        goto    menu_tm_time1           ;Jump to next key-in wait
;
;
;********************************************************************
;                      Clock Main Process
;********************************************************************
clock_main:
        btfss   rx_sts,0        ;Serial data received ?
        goto    clock_main1     ;No.
        call    sio_job         ;Serial communication processing
clock_main1:
        btfss   disp_sts        ;Just 0 seconds ?
        goto    clock_main2     ;No. Jump to decode check
        btfsc   disp_sts1       ;LCD displayed ?
        goto    clock_main4     ;Yes. Jump to return
        di                      ;Disable interrupt
        movf    PORTC,w         ;Read PORTC
        iorwf   portc_buf,w     ;Set ture time data
        movwf   PORTC           ;1:sec,2:minute,3:hour,4:day,5:month
        movf    porta_buf,w     ;Read timer data
        movwf   PORTA           ;0:TM1,1:TM2,2:TM3,3:TM4
        ei                      ;Enable interrupt
        btfss   stim_out        ;Time output suspension ?
        goto    clock_main5     ;No.
        call    sout_time       ;Send out Time to serial port
        bcf     stim_out        ;Clear time output suspension
clock_main5:
        btfss   disp_inh        ;Display inhibit ? (0:Disp/1:Stop)
        call    disp_main       ;No. Display time
        bsf     disp_sts1       ;Set LCD displayed flag
        bcf     dec_timing1     ;Clear decoded flag
        goto    clock_main4     ;Jump to return
clock_main2:
        btfss   dec_timing      ;Time decode timing ?(400mS passed ?)
        goto    clock_main4     ;No. Jump to return
        btfsc   dec_timing1     ;Decoded ?
        goto    clock_main4     ;No. Jump to return
        lcall   dec_time        ;Time decode
        lcall   inc_time        ;1 second + 1
        setpch
        btfsc   dec_time_1st    ;1st decode finished ?
        call    chk_timer       ;Yes. Check time of Timer
;
        movf    tm_1s,f         ;No. Read 1st of second
        jpnz    clock_main3     ;If 1st of second != 0, jump to clock_main3
        movf    tm_10s,f        ;Read 10th of second
        jpnz    clock_main3     ;If 10th of second != 0, jump to clock_main3
;*********************************************************
;I can't understand the role above 4 steps.
;It's the steps for true minute checking, but for what ?
;"clock_main3:" is executed without change of a situation
;on every condition.
;*********************************************************
clock_main3:
        bsf     dec_timing1     ;Set decoded flag
        bcf     disp_sts1       ;Clear displayed flag
clock_main4:
        return
;
;
;********************************************************************
;                          Time display
;********************************************************************
disp_main:
        clrf    xpos            ;Clear X position
        clrf    ypos            ;Clear Y position
        lcall   locate          ;Change cursor location
        setpch
        btfsc   det_mark_1st    ;1st marker detected ?
        goto    disp_main1      ;Yes. Jump to time decode check
        movlw   mes_search      ;No. Set "Mark searching  "
        goto    disp_main2      ;Jump to message out
disp_main1:
        btfsc   dec_time_1st    ;1st time decoded ?
        goto    disp_main3      ;Yes. Jump to Date display
        movlw   mes_dectime     ;No. Set "Time decoding   "
disp_main2:
        lcall   messout         ;Display message
        setpch
        goto    disp_main4      ;Jump to Time display
disp_main3:
        lcall   disp_date       ;Date display
disp_main4:
        lcall   disp_time       ;Time display
        setpch
        btfsc   dec_tco_err     ;Normal TCO(Time code Output) ?
        goto    disp_main5      ;No. Jump to NG display
        lcall   disp_ok         ;Yes. OK(O) display
        setpch
        goto    disp_main6      ;Jump to return
disp_main5
        lcall   disp_ng         ;NG(X) display
        setpch
disp_main6:
        return
;
;
;####################################################################
;
;                      Interruption processing
;
;####################################################################
int_vec:
        btfsc   PIR1,TMR2IF     ;timer2(100uS) int ?
        goto    timer_int       ;Yes.
        btfsc   PIR1,RCIF       ;RX int ?
        goto    sio_int         ;Yes.
;
timer_int:
        bcf     PIR1,TMR2IF     ;TRM2 int flag clear
        rbank1
        movf    delay_cnt100u,f ;Read 100uS delay counter
        jpz     timer_int0      ;If counter = 0, skip next
        decf    delay_cnt100u,f ;Counter - 1
;
timer_int0:                     ;(Every 100uS)
        incf    int_cnt,f       ;int_cnt + 1
        cp      int_cnt,100     ;10mS ?
        jpnz    int_vec1        ;No. Jump to end of interruption
        rbank0                  ;Yes. 10mS
        btfss   PORTC,1         ;True second = ON ?
        goto    timer_int00     ;No.
        incf    tled_cnt,f      ;Yes. LED turn on counter + 1
        cp      tled_cnt,10     ;100mS ?
        jpnz    timer_int00     ;No.
        movlw   b'00000001'
        andwf   PORTC,f         ;LED OFF except TCO
;
timer_int00:                    ;(Every 10mS)
        rbank1
        clrf    int_cnt         ;Clear int_cnt
        movf    delay_cnt10m,f  ;Read 10mS delay counter
        jpz     tco_chk         ;If counter = 0, skip next
        decf    delay_cnt10m,f  ;Counter - 1
;
;********************************************************************
;             TCO:Time Code Output check (Every 10mS)
;********************************************************************
tco_chk:
        rbank0
        btfsc   TCO                 ;TCO low ?
        goto    tco_chk6            ;No. TCO is high
        rbank1                      ;Yes. TCO is low
        movf    tco_sts,f           ;Read last look
        jpz     tco_chk5            ;Jump if last look is low(not fall edge)

;*** TCO fall edge detected
        rbank0
        bsf     tco_out             ;Set TCO LED ON (RC0)
        rbank1
        clrf    tco_sts             ;Set TCO low condition
        movf    tco_period,w        ;Read TCO period counter ??????
        movf    tco_cnt,w           ;Read TCO counter ?????
;
        rbank0
        clrf    tco_valid           ;TCO status clear
        rbank1
        cp      tco_period,tco_pl   ;TCO period check
        jpb     tco_chk4            ;Jump if TCO period < Lower limit(90)
        cp      tco_period,tco_ph+1 ;TCO period check
        jpnb    tco_chk4            ;Jump if TCO period >= Higher limit(111)
        rbank0
        bsf     tco_valid,tco_vp    ;Set TCO valid period flag
;
;*** Check Marker
        rbank1
        cp      tco_cnt,tco_ml      ;TCO mark check
        jpb     tco_chk1            ;Jump if TCO counter < Lower limit(8)
        cp      tco_cnt,tco_mh+1    ;TCO mark check
        jpnb    tco_chk1            ;Jump if TCO counter >= Higher limit(29)
        clrf    sync_cnt            ;Clear 0 code counter for Sync detect
        movlw   tco_m               ;Set TCO marker code(2)
        goto    tco_chk3
;
;*** Check 0 code
tco_chk1:
        cp      tco_cnt,tco_0l      ;TCO 0 check
        jpb     tco_chk2            ;Jump if TCO counter < Lower limit(70)
        cp      tco_cnt,tco_0h+1    ;TCO 0 check
        jpnb    tco_chk2            ;Jump if TCO counter >= Higher limit(89)
        incf    sync_cnt,f          ;0 code counter for Sync detect + 1
        movlw   tco_0               ;Set TCO 0 code(0)
        goto    tco_chk3
;
;*** Check 1 code
tco_chk2:
        cp      tco_cnt,tco_1l      ;TCO 1 check
        jpb     tco_chk4            ;Jump if TCO counter < Lower limit(40)
        cp      tco_cnt,tco_1h+1    ;TCO 1 check
        jpnb    tco_chk4            ;Jump if TCO counter >= Higher limit(61)
        clrf    sync_cnt            ;Clear 0 code counter for Sync detect
        movlw   tco_1               ;Set TCO 1 code(1)
;
tco_chk3:
        rbank0
        movwf   tco_code            ;Save TCO code
        btfsc   sync_sts            ;Check second sync flag
        goto    tco_chk30           ;If sync, jump to tco_chk30
        rbank1                      ;Not synchronized yet
        cp      sync_cnt,2          ;0 code check
        jpb     tco_chk30           ;Jump if 0 code < 3
        clrf    int_cnt10m          ;Clear 10mS interval counter
        clrf    int_cnt100m         ;Clear 100mS interval counter
        rbank0
        bsf     sync_sts            ;Set second sync flag
;
;*** 0 code less than 3
tco_chk30:
        rbank0
        bsf     tco_valid,tco_vc    ;Set TCO code valid bit
        rbank1
        movlw   1
        movwf   tco_period          ;Initialization of TCO period counter
        movwf   tco_cnt             ;Initialization of TCO counter
        goto    timer_int1
;
;*** TCO error
tco_chk4:
        clrf    sync_cnt            ;Clear 0 code counter for Sync detect
        clrf    tco_period          ;Clear TCO period counter
        clrf    tco_cnt             ;Clear TCO counter
        rbank0
        bsf     tco_valid,tco_err   ;Set TCO error flag
        clrf    tco_code            ;Clear TCO code
        goto    timer_int1
;
;*** TCO low is detected in the state of last look is low.
tco_chk5:
        clrf    tco_hicnt           ;Clear TCO high counter
        incf    tco_period,f        ;TCO period counter + 1
        cp      tco_period,tco_ph+1 ;TCO period check
        jpnb    tco_chk7            ;Jump TCO period >= Higher limit(111)
        incf    tco_cnt,f           ;TCO counter + 1
        cp      tco_cnt,tco_0h+1    ;TCO 0 check
        jpnb    tco_chk7            ;Jump if TCO counter >= Higher limit(89)
        goto    timer_int1
;
;*** TCO high is detected.
tco_chk6:
        bcf     tco_out             ;Set TCO LED OFF (RC0)
        rbank1
        incf    tco_hicnt,f         ;TCO high counter + 1
        incf    tco_period,f        ;TCO period counter + 1
        cp      tco_period,tco_ph+1 ;TCO period check
        jpnb    tco_chk7            ;Jump if TCO period >= Higher limit(111)
        bsf     tco_sts,0           ;Set TCO high flag
        goto    timer_int1
;
;*** TCO error
tco_chk7:
        clrf    tco_period          ;Clear TCO period counter
        clrf    tco_cnt             ;Clear TCO counter
        rbank0
        clrf    tco_code            ;Clear TCO code
        clrf    tco_valid           ;Clear TCO valid flag
        bsf     tco_valid,tco_err   ;Set TCO error flag
;
timer_int1:
        rbank1
        movf    int_cnt100m,f       ;100mS interval counter = 0 ?
        jpnz    timer_int2          ;No. Jump to timer_int2
        movf    int_cnt10m,f        ;Yes. 10mS interval counter = 0 ?
        jpnz    timer_int2          ; No. Jump to timer_int2
        rbank0                      ; Yes.
        bsf     disp_sts            ;Set LCD display timing flag(Every 0 sec)
        bcf     dec_timing          ;Clear Decode timing flag
        goto    timer_int3
timer_int2:
        cp      int_cnt100m,4       ;Check 100mS interval counter
        jpnz    timer_int3          ;Jump if 100mS interval counter != 4
        movf    int_cnt10m,f        ;10mS interval counter = 0 ?
        jpnz    timer_int3          ;Jump if 10mS interval counter != 0
        rbank0
        bsf     dec_timing          ;Set Decode timing flag(Every 400mS)
        bcf     disp_sts            ;Cleat LCD display timing flag
timer_int3:
        rbank1
        incf    int_cnt10m,f        ;10mS interval counter + 1
        cp      int_cnt10m,10       ;Check 10mS interval counter
        jpnz    timer_int4          ;Jump if 10mS interval counter != 10
        rbank0
        movf    k_tmout_cnt,f       ;key time-out counter = 0 ?
        jpz     timer_int5          ;Jump if key time-out counter = 0
        decf    k_tmout_cnt,f       ;key time-out counter - 1
timer_int5:
        rbank1
        clrf    int_cnt10m          ;Clear 10mS interval counter
        incf    int_cnt100m,f       ;100mS interval counter + 1
        cp      int_cnt100m,10      ;Check 100mS interval counter
        jpnz    timer_int4          ;Jump if 100mS interval counter != 10
        clrf    int_cnt100m         ;Clear 100mS interval counter
;
;********************************************************************
;                     Key input process
;********************************************************************
timer_int4:
        rbank0
        btfsc   key_sts,0           ;Key pressed ?
        goto    int_vec1            ;Yes. Jump to return
        rbank1                      ;No.
        movlw   b'11111100'         ;RB0,1=OUT RB2-7=IN
        movwf   TRIS_B              ;Set TRISB register
        rbank0
        nop                         ;Make timing (3.124uS)
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        comf    PORT_B,w            ;Complement 0/1 input data
        andlw   b'11111100'         ;Pick up RB2-7
        rbank1
        movwf   ikey_dat1           ;Save key input data
        movlw   b'00001100'         ;RB0,1,4-7=OUT RB2,3=IN
        movwf   TRIS_B              ;Set TRISB register
        movf    ikey_dat1,f         ;Key pressed ?
        jpz     kswchkend           ;No. Jump key check end
;k-sw check
        incf    key_phs,f           ;Yes. Key phase + 1
        jpnz    kswchk01            ;Result = 0 ?
        decf    key_phs,f           ;Yes. Stop key phase
kswchk01:
        cp      key_phs,1           ;No. key phase = 1 ?
        jpnz    kswchk1             ;if phase != 1 then jump
;
;*** Check 1st pressed key
        clrf    ikey_dat            ;Clear key number
        clrf    ikey_dat0           ;Clear key scan positon
        bsf     ikey_dat0,2         ;Set key scan bit bit
        clrc                        ;Clear carry flag
kswchk02:
        movf    ikey_dat0,w         ;Read key scan positon
        andwf   ikey_dat1,w         ;Key ON ?
        jpnz    int_vec1            ;Yes. Jump to return
        incf    ikey_dat,f          ;No. key number + 1
        rlf     ikey_dat0,f         ;Set next key scan position
        jpnc    kswchk02            ;If not end, next key check
        goto    kswchkend           ;Jump to keyend
;
;*** Continuous operation check of the key
kswchk1:
        movf    ikey_dat0,w         ;Read key scan positon
        andwf   ikey_dat1,w         ;Key ON ?
        jpz     kswchkend           ;No. Jump to keyend
        cp      key_phs,2           ;Key phase = 2 ?
        jpnz    kswchk3             ;No. jump to kswchk3
        movf    ikey_dat,w          ;Yes. Read key number
        rbank0
        movwf   key_dat             ;Save key number
        bsf     key_sts,0           ;Set key status
        goto    int_vec1            ;Jump to return
kswchk3:
        cp      key_phs,rep_start   ;key repeate start timng(500mS) ?
        jpb     int_vec1            ;If key_phs<rep_start, jump to return
        jpnb    kswchk4             ;If key_phs>=rep_start, jump to kswchk4
;*********************************************************
;I think the following steps are not executed.
;And moreover the same processing as the following is used for some parts.
;They can be unified.
;*********************************************************
        clrf    rep_cnt             ;Clear key repeate counter
        movf    ikey_dat,w          ;Read key number
        rbank0
        movwf   key_dat             ;Save key number
        bsf     key_sts,0           ;Set key status
        goto    int_vec1            ;Jump to return
;
kswchk4:
        cp      key_phs,rep2_delay  ;Key 2nd delay timing(2.4S) ?
        jpz     kswchk5             ;If key_phs=rep2_delay, jump to kswchk5
        jpnb    kswchk6             ;If key_phs>=rep2_delay, jump to kswchk6
                                    ;If key_phs<rep2_delay, go to next
        incf    rep_cnt,f           ;key repeate counter + 1
;
        cp      rep_cnt,rep1_period ;Key 1 period timing(300mS) ?
        jpnz    int_vec1            ;If not rep1_period, jump to return
                                    ;If key_phs=rep1_period, go to next
        clrf    rep_cnt             ;Clear key repeate counter
        movf    ikey_dat,w          ;Read key number
        rbank0
        movwf   key_dat             ;Save key number
        bsf     key_sts,0           ;Set key status
        goto    int_vec1            ;Jump to return
;
kswchk5:
        clrf    rep_cnt             ;Clear key repeate counter
        movf    ikey_dat,w          ;Read key number
        rbank0
        movwf   key_dat             ;Save key number
        bsf     key_sts,0           ;Set key status
        goto    int_vec1            ;Jump to return
;
kswchk6:
        incf    rep_cnt,f           ;key repeate counter + 1
        cp      rep_cnt,rep2_period ;Key 2 period timing(100mS) ?
        jpnz    int_vec1            ;If not rep2_period, jump to return
                                    ;If key_phs=rep2_period, go to next
        clrf    rep_cnt             ;Clear key repeate counter
        movf    ikey_dat,w          ;Read key number
        rbank0
        movwf   key_dat             ;Save key number
        bsf     key_sts,0           ;Set key status
        goto    int_vec1            ;Jump to return
;
kswchkend:
        clrf    key_phs             ;Clear key phase data
        clrf    rep_cnt             ;Clear key repeate counter
        goto    int_vec1            ;Jump to return
;
;********************************************************************
;                       Serial port process
;********************************************************************
sio_int:
        bcf     PIR1,RCIF           ;Clear rx int flag
        btfss   RCSTA,FERR          ;Frame error ?
        goto    sio_int1            ;No. Jump to sio_int1
        movf    RCREG,f             ;Yes. Received data deletion
        goto    int_vec1            ;Jump to return
sio_int1:
        btfss   RCSTA,OERR          ;Over run ?
        goto    sio_int2            ;No. Jump to sio_int2
        bcf     RCSTA,CREN          ;Yes. Clear error flag
        bsf     RCSTA,CREN          ;Set Continuous Receive Enable bit
        goto    int_vec1            ;Jump to return
sio_int2:
        movf    rx_ptr,w            ;Read received data pointer
        movwf   FSR                 ;Set received data pointer
        movf    RCREG,w             ;Read received data
        btfsc   rx_sts,0            ;Before data processed ?
        goto    int_vec1            ;No. Jump to return
        movwf   INDF                ;Yes. Readable
        cp      INDF,0x0a           ;LF ?
        jpnz    sio_int3            ;No.
        goto    sio_int6            ;Yes. Jump to return
sio_int3:
        cp      INDF,0x0d           ;CR ?
        jpnz    sio_int4            ;No.
        bsf     rx_sts,0            ;Yes. Set received flag
        goto    sio_int6            ;Jump to return
sio_int4:
        incf    rx_cc,f             ;Received data counter + 1
        cp      rx_cc,RxBfLen       ;Counter < Receive buffer length(28) ?
        jpb     sio_int5            ;Yes. Jump to sio_int5
        decf    rx_cc,f             ;No. Counter - 1
        goto    sio_int6            ;Jump to return
sio_int5:
        incf    rx_ptr,f            ;Received data pointer + 1
sio_int6:
;
;
;********************************************************************
;                      End of interruption process
;********************************************************************
int_vec1:
        rbank0
        movf    tmp_FSR,w
        movwf   FSR                 ;Resave FSR register
        movf    tmp_pclath,w
        movwf   PCLATH              ;Resave PCLATH register
        swapf   tmp_sts,w
        movwf   STATUS              ;Resave STATUS register
        swapf   tmp_w,f
        swapf   tmp_w,w             ;Resave W register
        retfie                      ;End of interruption
;
        end